@@ -1,5 +1,6 @@
 from __future__ import annotations
 
+import atexit
 import contextlib
 import io
 import threading
@@ -289,6 +290,13 @@ class CachingFileManager(FileManager):
         )
 
 
+@atexit.register
+def _remove_del_method():
+    # We don't need to close unclosed files at program exit, and may not be able
+    # to, because Python is cleaning up imports / globals.
+    del CachingFileManager.__del__
+
+
 class _RefCounter:
     """Class for keeping track of reference counts."""
 
