@@ -2,7 +2,8 @@ from __future__ import annotations
 
 import threading
 from collections import OrderedDict
-from typing import Any, Callable, Iterator, MutableMapping, TypeVar
+from collections.abc import Iterator, MutableMapping
+from typing import Any, Callable, TypeVar
 
 K = TypeVar("K")
 V = TypeVar("V")
