@@ -1,6 +1,7 @@
 """
 Functions for applying functions that act on arrays to xarray's labeled data.
 """
+
 from __future__ import annotations
 
 import functools
@@ -8,8 +9,8 @@ import itertools
 import operator
 import warnings
 from collections import Counter
-from collections.abc import Hashable, Iterable, Mapping, Sequence, Set
-from typing import TYPE_CHECKING, Any, Callable, Literal, TypeVar, Union, overload
+from collections.abc import Hashable, Iterable, Iterator, Mapping, Sequence, Set
+from typing import TYPE_CHECKING, Any, Callable, Literal, TypeVar, Union, cast, overload
 
 import numpy as np
 
@@ -17,14 +18,16 @@ from xarray.core import dtypes, duck_array_ops, utils
 from xarray.core.alignment import align, deep_align
 from xarray.core.common import zeros_like
 from xarray.core.duck_array_ops import datetime_to_numeric
+from xarray.core.formatting import limit_lines
 from xarray.core.indexes import Index, filter_indexes_from_coords
 from xarray.core.merge import merge_attrs, merge_coordinates_without_align
 from xarray.core.options import OPTIONS, _get_keep_attrs
-from xarray.core.parallelcompat import get_chunked_array_type
-from xarray.core.pycompat import is_chunked_array, is_duck_dask_array
 from xarray.core.types import Dims, T_DataArray
-from xarray.core.utils import is_dict_like, is_scalar
+from xarray.core.utils import is_dict_like, is_duck_dask_array, is_scalar, parse_dims
 from xarray.core.variable import Variable
+from xarray.namedarray.parallelcompat import get_chunked_array_type
+from xarray.namedarray.pycompat import is_chunked_array
+from xarray.util.deprecation_helpers import deprecate_dims
 
 if TYPE_CHECKING:
     from xarray.core.coordinates import Coordinates
@@ -32,6 +35,8 @@ if TYPE_CHECKING:
     from xarray.core.dataset import Dataset
     from xarray.core.types import CombineAttrsOptions, JoinOptions
 
+    MissingCoreDimOptions = Literal["raise", "copy", "drop"]
+
 _NO_FILL_VALUE = utils.ReprObject("<no-fill-value>")
 _DEFAULT_NAME = utils.ReprObject("<default-name>")
 _JOINS_WITHOUT_FILL_VALUES = frozenset({"inner", "exact"})
@@ -42,6 +47,7 @@ def _first_of_type(args, kind):
     for arg in args:
         if isinstance(arg, kind):
             return arg
+
     raise ValueError("This should be unreachable.")
 
 
@@ -159,7 +165,7 @@ class _UFuncSignature:
         if exclude_dims:
             exclude_dims = [self.dims_map[dim] for dim in exclude_dims]
 
-            counter = Counter()
+            counter: Counter = Counter()
 
             def _enumerate(dim):
                 if dim in exclude_dims:
@@ -285,8 +291,14 @@ def apply_dataarray_vfunc(
     from xarray.core.dataarray import DataArray
 
     if len(args) > 1:
-        args = deep_align(
-            args, join=join, copy=False, exclude=exclude_dims, raise_on_invalid=False
+        args = tuple(
+            deep_align(
+                args,
+                join=join,
+                copy=False,
+                exclude=exclude_dims,
+                raise_on_invalid=False,
+            )
         )
 
     objs = _all_of_type(args, DataArray)
@@ -347,7 +359,7 @@ def assert_and_return_exact_match(all_keys):
         if keys != first_keys:
             raise ValueError(
                 "exact match required for all data variable names, "
-                f"but {keys!r} != {first_keys!r}"
+                f"but {list(keys)} != {list(first_keys)}: {set(keys) ^ set(first_keys)} are not in both."
             )
     return first_keys
 
@@ -376,7 +388,7 @@ def collect_dict_values(
     ]
 
 
-def _as_variables_or_variable(arg):
+def _as_variables_or_variable(arg) -> Variable | tuple[Variable]:
     try:
         return arg.variables
     except AttributeError:
@@ -396,8 +408,39 @@ def _unpack_dict_tuples(
     return out
 
 
+def _check_core_dims(signature, variable_args, name):
+    """
+    Check if an arg has all the core dims required by the signature.
+
+    Slightly awkward design, of returning the error message. But we want to
+    give a detailed error message, which requires inspecting the variable in
+    the inner loop.
+    """
+    missing = []
+    for i, (core_dims, variable_arg) in enumerate(
+        zip(signature.input_core_dims, variable_args)
+    ):
+        # Check whether all the dims are on the variable. Note that we need the
+        # `hasattr` to check for a dims property, to protect against the case where
+        # a numpy array is passed in.
+        if hasattr(variable_arg, "dims") and set(core_dims) - set(variable_arg.dims):
+            missing += [[i, variable_arg, core_dims]]
+    if missing:
+        message = ""
+        for i, variable_arg, core_dims in missing:
+            message += f"Missing core dims {set(core_dims) - set(variable_arg.dims)} from arg number {i + 1} on a variable named `{name}`:\n{variable_arg}\n\n"
+        message += "Either add the core dimension, or if passing a dataset alternatively pass `on_missing_core_dim` as `copy` or `drop`. "
+        return message
+    return True
+
+
 def apply_dict_of_variables_vfunc(
-    func, *args, signature: _UFuncSignature, join="inner", fill_value=None
+    func,
+    *args,
+    signature: _UFuncSignature,
+    join="inner",
+    fill_value=None,
+    on_missing_core_dim: MissingCoreDimOptions = "raise",
 ):
     """Apply a variable level function over dicts of DataArray, DataArray,
     Variable and ndarray objects.
@@ -408,7 +451,20 @@ def apply_dict_of_variables_vfunc(
 
     result_vars = {}
     for name, variable_args in zip(names, grouped_by_name):
-        result_vars[name] = func(*variable_args)
+        core_dim_present = _check_core_dims(signature, variable_args, name)
+        if core_dim_present is True:
+            result_vars[name] = func(*variable_args)
+        else:
+            if on_missing_core_dim == "raise":
+                raise ValueError(core_dim_present)
+            elif on_missing_core_dim == "copy":
+                result_vars[name] = variable_args[0]
+            elif on_missing_core_dim == "drop":
+                pass
+            else:
+                raise ValueError(
+                    f"Invalid value for `on_missing_core_dim`: {on_missing_core_dim!r}"
+                )
 
     if signature.num_outputs > 1:
         return _unpack_dict_tuples(result_vars, signature.num_outputs)
@@ -441,6 +497,7 @@ def apply_dataset_vfunc(
     fill_value=_NO_FILL_VALUE,
     exclude_dims=frozenset(),
     keep_attrs="override",
+    on_missing_core_dim: MissingCoreDimOptions = "raise",
 ) -> Dataset | tuple[Dataset, ...]:
     """Apply a variable level function over Dataset, dict of DataArray,
     DataArray, Variable and/or ndarray objects.
@@ -457,8 +514,14 @@ def apply_dataset_vfunc(
     objs = _all_of_type(args, Dataset)
 
     if len(args) > 1:
-        args = deep_align(
-            args, join=join, copy=False, exclude=exclude_dims, raise_on_invalid=False
+        args = tuple(
+            deep_align(
+                args,
+                join=join,
+                copy=False,
+                exclude=exclude_dims,
+                raise_on_invalid=False,
+            )
         )
 
     list_of_coords, list_of_indexes = build_output_coords_and_indexes(
@@ -467,7 +530,12 @@ def apply_dataset_vfunc(
     args = tuple(getattr(arg, "data_vars", arg) for arg in args)
 
     result_vars = apply_dict_of_variables_vfunc(
-        func, *args, signature=signature, join=dataset_join, fill_value=fill_value
+        func,
+        *args,
+        signature=signature,
+        join=dataset_join,
+        fill_value=fill_value,
+        on_missing_core_dim=on_missing_core_dim,
     )
 
     out: Dataset | tuple[Dataset, ...]
@@ -517,7 +585,7 @@ def apply_groupby_func(func, *args):
     assert groupbys, "must have at least one groupby to iterate over"
     first_groupby = groupbys[0]
     (grouper,) = first_groupby.groupers
-    if any(not grouper.group.equals(gb.groupers[0].group) for gb in groupbys[1:]):
+    if any(not grouper.group.equals(gb.groupers[0].group) for gb in groupbys[1:]):  # type: ignore[union-attr]
         raise ValueError(
             "apply_ufunc can only perform operations over "
             "multiple GroupBy objects at once if they are all "
@@ -529,6 +597,7 @@ def apply_groupby_func(func, *args):
 
     iterators = []
     for arg in args:
+        iterator: Iterator[Any]
         if isinstance(arg, GroupBy):
             iterator = (value for _, value in arg)
         elif hasattr(arg, "dims") and grouped_dim in arg.dims:
@@ -543,9 +612,9 @@ def apply_groupby_func(func, *args):
             iterator = itertools.repeat(arg)
         iterators.append(iterator)
 
-    applied = (func(*zipped_args) for zipped_args in zip(*iterators))
+    applied: Iterator = (func(*zipped_args) for zipped_args in zip(*iterators))
     applied_example, applied = peek_at(applied)
-    combine = first_groupby._combine
+    combine = first_groupby._combine  # type: ignore[attr-defined]
     if isinstance(applied_example, tuple):
         combined = tuple(combine(output) for output in zip(*applied))
     else:
@@ -595,17 +664,9 @@ def broadcast_compat_data(
         return data
 
     set_old_dims = set(old_dims)
-    missing_core_dims = [d for d in core_dims if d not in set_old_dims]
-    if missing_core_dims:
-        raise ValueError(
-            "operand to apply_ufunc has required core dimensions {}, but "
-            "some of these dimensions are absent on an input variable: {}".format(
-                list(core_dims), missing_core_dims
-            )
-        )
-
     set_new_dims = set(new_dims)
     unexpected_dims = [d for d in old_dims if d not in set_new_dims]
+
     if unexpected_dims:
         raise ValueError(
             "operand to apply_ufunc encountered unexpected "
@@ -659,6 +720,7 @@ def apply_variable_ufunc(
     dask_gufunc_kwargs=None,
 ) -> Variable | tuple[Variable, ...]:
     """Apply a ndarray level function over Variable and/or ndarray objects."""
+    from xarray.core.formatting import short_array_repr
     from xarray.core.variable import Variable, as_compatible_data
 
     dim_sizes = unified_dim_sizes(
@@ -670,9 +732,11 @@ def apply_variable_ufunc(
     output_dims = [broadcast_dims + out for out in signature.output_core_dims]
 
     input_data = [
-        broadcast_compat_data(arg, broadcast_dims, core_dims)
-        if isinstance(arg, Variable)
-        else arg
+        (
+            broadcast_compat_data(arg, broadcast_dims, core_dims)
+            if isinstance(arg, Variable)
+            else arg
+        )
         for arg, core_dims in zip(args, signature.input_core_dims)
     ]
 
@@ -750,7 +814,7 @@ def apply_variable_ufunc(
             pass
         else:
             raise ValueError(
-                "unknown setting for chunked array handling in " f"apply_ufunc: {dask}"
+                f"unknown setting for chunked array handling in apply_ufunc: {dask}"
             )
     else:
         if vectorize:
@@ -766,11 +830,11 @@ def apply_variable_ufunc(
         not isinstance(result_data, tuple) or len(result_data) != signature.num_outputs
     ):
         raise ValueError(
-            "applied function does not have the number of "
-            "outputs specified in the ufunc signature. "
-            "Result is not a tuple of {} elements: {!r}".format(
-                signature.num_outputs, result_data
-            )
+            f"applied function does not have the number of "
+            f"outputs specified in the ufunc signature. "
+            f"Received a {type(result_data)} with {len(result_data)} elements. "
+            f"Expected a tuple of {signature.num_outputs} elements:\n\n"
+            f"{limit_lines(repr(result_data), limit=10)}"
         )
 
     objs = _all_of_type(args, Variable)
@@ -784,21 +848,22 @@ def apply_variable_ufunc(
         data = as_compatible_data(data)
         if data.ndim != len(dims):
             raise ValueError(
-                "applied function returned data with unexpected "
+                "applied function returned data with an unexpected "
                 f"number of dimensions. Received {data.ndim} dimension(s) but "
-                f"expected {len(dims)} dimensions with names: {dims!r}"
+                f"expected {len(dims)} dimensions with names {dims!r}, from:\n\n"
+                f"{short_array_repr(data)}"
             )
 
         var = Variable(dims, data, fastpath=True)
         for dim, new_size in var.sizes.items():
             if dim in dim_sizes and new_size != dim_sizes[dim]:
                 raise ValueError(
-                    "size of dimension {!r} on inputs was unexpectedly "
-                    "changed by applied function from {} to {}. Only "
+                    f"size of dimension '{dim}' on inputs was unexpectedly "
+                    f"changed by applied function from {dim_sizes[dim]} to {new_size}. Only "
                     "dimensions specified in ``exclude_dims`` with "
-                    "xarray.apply_ufunc are allowed to change size.".format(
-                        dim, dim_sizes[dim], new_size
-                    )
+                    "xarray.apply_ufunc are allowed to change size. "
+                    "The data returned was:\n\n"
+                    f"{short_array_repr(data)}"
                 )
 
         var.attrs = attrs
@@ -845,11 +910,12 @@ def apply_ufunc(
     dataset_fill_value: object = _NO_FILL_VALUE,
     keep_attrs: bool | str | None = None,
     kwargs: Mapping | None = None,
-    dask: str = "forbidden",
+    dask: Literal["forbidden", "allowed", "parallelized"] = "forbidden",
     output_dtypes: Sequence | None = None,
     output_sizes: Mapping[Any, int] | None = None,
     meta: Any = None,
     dask_gufunc_kwargs: dict[str, Any] | None = None,
+    on_missing_core_dim: MissingCoreDimOptions = "raise",
 ) -> Any:
     """Apply a vectorized function for unlabeled arrays on xarray objects.
 
@@ -963,6 +1029,8 @@ def apply_ufunc(
         :py:func:`dask.array.apply_gufunc`. ``meta`` should be given in the
         ``dask_gufunc_kwargs`` parameter . It will be removed as direct parameter
         a future version.
+    on_missing_core_dim : {"raise", "copy", "drop"}, default: "raise"
+        How to handle missing core dimensions on input variables.
 
     Returns
     -------
@@ -991,10 +1059,10 @@ def apply_ufunc(
 
     >>> array = xr.DataArray([1, 2, 3], coords=[("x", [0.1, 0.2, 0.3])])
     >>> magnitude(array, -array)
-    <xarray.DataArray (x: 3)>
+    <xarray.DataArray (x: 3)> Size: 24B
     array([1.41421356, 2.82842712, 4.24264069])
     Coordinates:
-      * x        (x) float64 0.1 0.2 0.3
+      * x        (x) float64 24B 0.1 0.2 0.3
 
     Plain scalars, numpy arrays and a mix of these with xarray objects is also
     supported:
@@ -1004,10 +1072,10 @@ def apply_ufunc(
     >>> magnitude(3, np.array([0, 4]))
     array([3., 5.])
     >>> magnitude(array, 0)
-    <xarray.DataArray (x: 3)>
+    <xarray.DataArray (x: 3)> Size: 24B
     array([1., 2., 3.])
     Coordinates:
-      * x        (x) float64 0.1 0.2 0.3
+      * x        (x) float64 24B 0.1 0.2 0.3
 
     Other examples of how you could use ``apply_ufunc`` to write functions to
     (very nearly) replicate existing xarray functionality:
@@ -1077,9 +1145,13 @@ def apply_ufunc(
     numba.guvectorize
     dask.array.apply_gufunc
     xarray.map_blocks
+
     :ref:`dask.automatic-parallelization`
         User guide describing :py:func:`apply_ufunc` and :py:func:`map_blocks`.
 
+    :doc:`xarray-tutorial:advanced/apply_ufunc/apply_ufunc`
+        Advanced Tutorial on applying numpy function using :py:func:`apply_ufunc`
+
     References
     ----------
     .. [1] https://numpy.org/doc/stable/reference/ufuncs.html
@@ -1191,6 +1263,7 @@ def apply_ufunc(
             dataset_join=dataset_join,
             fill_value=dataset_fill_value,
             keep_attrs=keep_attrs,
+            on_missing_core_dim=on_missing_core_dim,
         )
     # feed DataArray apply_variable_ufunc through apply_dataarray_vfunc
     elif any(isinstance(a, DataArray) for a in args):
@@ -1211,7 +1284,11 @@ def apply_ufunc(
 
 
 def cov(
-    da_a: T_DataArray, da_b: T_DataArray, dim: Dims = None, ddof: int = 1
+    da_a: T_DataArray,
+    da_b: T_DataArray,
+    dim: Dims = None,
+    ddof: int = 1,
+    weights: T_DataArray | None = None,
 ) -> T_DataArray:
     """
     Compute covariance between two DataArray objects along a shared dimension.
@@ -1227,6 +1304,8 @@ def cov(
     ddof : int, default: 1
         If ddof=1, covariance is normalized by N-1, giving an unbiased estimate,
         else normalization is by N.
+    weights : DataArray, optional
+        Array of weights.
 
     Returns
     -------
@@ -1249,13 +1328,13 @@ def cov(
     ...     ],
     ... )
     >>> da_a
-    <xarray.DataArray (space: 3, time: 3)>
+    <xarray.DataArray (space: 3, time: 3)> Size: 72B
     array([[1. , 2. , 3. ],
            [0.1, 0.2, 0.3],
            [3.2, 0.6, 1.8]])
     Coordinates:
-      * space    (space) <U2 'IA' 'IL' 'IN'
-      * time     (time) datetime64[ns] 2000-01-01 2000-01-02 2000-01-03
+      * space    (space) <U2 24B 'IA' 'IL' 'IN'
+      * time     (time) datetime64[ns] 24B 2000-01-01 2000-01-02 2000-01-03
     >>> da_b = DataArray(
     ...     np.array([[0.2, 0.4, 0.6], [15, 10, 5], [3.2, 0.6, 1.8]]),
     ...     dims=("space", "time"),
@@ -1265,34 +1344,58 @@ def cov(
     ...     ],
     ... )
     >>> da_b
-    <xarray.DataArray (space: 3, time: 3)>
+    <xarray.DataArray (space: 3, time: 3)> Size: 72B
     array([[ 0.2,  0.4,  0.6],
            [15. , 10. ,  5. ],
            [ 3.2,  0.6,  1.8]])
     Coordinates:
-      * space    (space) <U2 'IA' 'IL' 'IN'
-      * time     (time) datetime64[ns] 2000-01-01 2000-01-02 2000-01-03
+      * space    (space) <U2 24B 'IA' 'IL' 'IN'
+      * time     (time) datetime64[ns] 24B 2000-01-01 2000-01-02 2000-01-03
     >>> xr.cov(da_a, da_b)
-    <xarray.DataArray ()>
+    <xarray.DataArray ()> Size: 8B
     array(-3.53055556)
     >>> xr.cov(da_a, da_b, dim="time")
-    <xarray.DataArray (space: 3)>
+    <xarray.DataArray (space: 3)> Size: 24B
     array([ 0.2       , -0.5       ,  1.69333333])
     Coordinates:
-      * space    (space) <U2 'IA' 'IL' 'IN'
+      * space    (space) <U2 24B 'IA' 'IL' 'IN'
+    >>> weights = DataArray(
+    ...     [4, 2, 1],
+    ...     dims=("space"),
+    ...     coords=[
+    ...         ("space", ["IA", "IL", "IN"]),
+    ...     ],
+    ... )
+    >>> weights
+    <xarray.DataArray (space: 3)> Size: 24B
+    array([4, 2, 1])
+    Coordinates:
+      * space    (space) <U2 24B 'IA' 'IL' 'IN'
+    >>> xr.cov(da_a, da_b, dim="space", weights=weights)
+    <xarray.DataArray (time: 3)> Size: 24B
+    array([-4.69346939, -4.49632653, -3.37959184])
+    Coordinates:
+      * time     (time) datetime64[ns] 24B 2000-01-01 2000-01-02 2000-01-03
     """
     from xarray.core.dataarray import DataArray
 
     if any(not isinstance(arr, DataArray) for arr in [da_a, da_b]):
         raise TypeError(
             "Only xr.DataArray is supported."
-            "Given {}.".format([type(arr) for arr in [da_a, da_b]])
+            f"Given {[type(arr) for arr in [da_a, da_b]]}."
         )
-
-    return _cov_corr(da_a, da_b, dim=dim, ddof=ddof, method="cov")
+    if weights is not None:
+        if not isinstance(weights, DataArray):
+            raise TypeError(f"Only xr.DataArray is supported. Given {type(weights)}.")
+    return _cov_corr(da_a, da_b, weights=weights, dim=dim, ddof=ddof, method="cov")
 
 
-def corr(da_a: T_DataArray, da_b: T_DataArray, dim: Dims = None) -> T_DataArray:
+def corr(
+    da_a: T_DataArray,
+    da_b: T_DataArray,
+    dim: Dims = None,
+    weights: T_DataArray | None = None,
+) -> T_DataArray:
     """
     Compute the Pearson correlation coefficient between
     two DataArray objects along a shared dimension.
@@ -1305,6 +1408,8 @@ def corr(da_a: T_DataArray, da_b: T_DataArray, dim: Dims = None) -> T_DataArray:
         Array to compute.
     dim : str, iterable of hashable, "..." or None, optional
         The dimension along which the correlation will be computed
+    weights : DataArray, optional
+        Array of weights.
 
     Returns
     -------
@@ -1327,13 +1432,13 @@ def corr(da_a: T_DataArray, da_b: T_DataArray, dim: Dims = None) -> T_DataArray:
     ...     ],
     ... )
     >>> da_a
-    <xarray.DataArray (space: 3, time: 3)>
+    <xarray.DataArray (space: 3, time: 3)> Size: 72B
     array([[1. , 2. , 3. ],
            [0.1, 0.2, 0.3],
            [3.2, 0.6, 1.8]])
     Coordinates:
-      * space    (space) <U2 'IA' 'IL' 'IN'
-      * time     (time) datetime64[ns] 2000-01-01 2000-01-02 2000-01-03
+      * space    (space) <U2 24B 'IA' 'IL' 'IN'
+      * time     (time) datetime64[ns] 24B 2000-01-01 2000-01-02 2000-01-03
     >>> da_b = DataArray(
     ...     np.array([[0.2, 0.4, 0.6], [15, 10, 5], [3.2, 0.6, 1.8]]),
     ...     dims=("space", "time"),
@@ -1343,36 +1448,56 @@ def corr(da_a: T_DataArray, da_b: T_DataArray, dim: Dims = None) -> T_DataArray:
     ...     ],
     ... )
     >>> da_b
-    <xarray.DataArray (space: 3, time: 3)>
+    <xarray.DataArray (space: 3, time: 3)> Size: 72B
     array([[ 0.2,  0.4,  0.6],
            [15. , 10. ,  5. ],
            [ 3.2,  0.6,  1.8]])
     Coordinates:
-      * space    (space) <U2 'IA' 'IL' 'IN'
-      * time     (time) datetime64[ns] 2000-01-01 2000-01-02 2000-01-03
+      * space    (space) <U2 24B 'IA' 'IL' 'IN'
+      * time     (time) datetime64[ns] 24B 2000-01-01 2000-01-02 2000-01-03
     >>> xr.corr(da_a, da_b)
-    <xarray.DataArray ()>
+    <xarray.DataArray ()> Size: 8B
     array(-0.57087777)
     >>> xr.corr(da_a, da_b, dim="time")
-    <xarray.DataArray (space: 3)>
+    <xarray.DataArray (space: 3)> Size: 24B
     array([ 1., -1.,  1.])
     Coordinates:
-      * space    (space) <U2 'IA' 'IL' 'IN'
+      * space    (space) <U2 24B 'IA' 'IL' 'IN'
+    >>> weights = DataArray(
+    ...     [4, 2, 1],
+    ...     dims=("space"),
+    ...     coords=[
+    ...         ("space", ["IA", "IL", "IN"]),
+    ...     ],
+    ... )
+    >>> weights
+    <xarray.DataArray (space: 3)> Size: 24B
+    array([4, 2, 1])
+    Coordinates:
+      * space    (space) <U2 24B 'IA' 'IL' 'IN'
+    >>> xr.corr(da_a, da_b, dim="space", weights=weights)
+    <xarray.DataArray (time: 3)> Size: 24B
+    array([-0.50240504, -0.83215028, -0.99057446])
+    Coordinates:
+      * time     (time) datetime64[ns] 24B 2000-01-01 2000-01-02 2000-01-03
     """
     from xarray.core.dataarray import DataArray
 
     if any(not isinstance(arr, DataArray) for arr in [da_a, da_b]):
         raise TypeError(
             "Only xr.DataArray is supported."
-            "Given {}.".format([type(arr) for arr in [da_a, da_b]])
+            f"Given {[type(arr) for arr in [da_a, da_b]]}."
         )
-
-    return _cov_corr(da_a, da_b, dim=dim, method="corr")
+    if weights is not None:
+        if not isinstance(weights, DataArray):
+            raise TypeError(f"Only xr.DataArray is supported. Given {type(weights)}.")
+    return _cov_corr(da_a, da_b, weights=weights, dim=dim, method="corr")
 
 
 def _cov_corr(
     da_a: T_DataArray,
     da_b: T_DataArray,
+    weights: T_DataArray | None = None,
     dim: Dims = None,
     ddof: int = 0,
     method: Literal["cov", "corr", None] = None,
@@ -1388,28 +1513,46 @@ def _cov_corr(
     valid_values = da_a.notnull() & da_b.notnull()
     da_a = da_a.where(valid_values)
     da_b = da_b.where(valid_values)
-    valid_count = valid_values.sum(dim) - ddof
 
     # 3. Detrend along the given dim
-    demeaned_da_a = da_a - da_a.mean(dim=dim)
-    demeaned_da_b = da_b - da_b.mean(dim=dim)
+    if weights is not None:
+        demeaned_da_a = da_a - da_a.weighted(weights).mean(dim=dim)
+        demeaned_da_b = da_b - da_b.weighted(weights).mean(dim=dim)
+    else:
+        demeaned_da_a = da_a - da_a.mean(dim=dim)
+        demeaned_da_b = da_b - da_b.mean(dim=dim)
 
     # 4. Compute covariance along the given dim
     # N.B. `skipna=True` is required or auto-covariance is computed incorrectly. E.g.
     # Try xr.cov(da,da) for da = xr.DataArray([[1, 2], [1, np.nan]], dims=["x", "time"])
-    cov = (demeaned_da_a.conj() * demeaned_da_b).sum(
-        dim=dim, skipna=True, min_count=1
-    ) / (valid_count)
+    if weights is not None:
+        cov = (
+            (demeaned_da_a.conj() * demeaned_da_b)
+            .weighted(weights)
+            .mean(dim=dim, skipna=True)
+        )
+    else:
+        cov = (demeaned_da_a.conj() * demeaned_da_b).mean(dim=dim, skipna=True)
 
     if method == "cov":
-        return cov  # type: ignore[return-value]
+        # Adjust covariance for degrees of freedom
+        valid_count = valid_values.sum(dim)
+        adjust = valid_count / (valid_count - ddof)
+        # I think the cast is required because of `T_DataArray` + `T_Xarray` (would be
+        # the same with `T_DatasetOrArray`)
+        # https://github.com/pydata/xarray/pull/8384#issuecomment-1784228026
+        return cast(T_DataArray, cov * adjust)
 
     else:
-        # compute std + corr
-        da_a_std = da_a.std(dim=dim)
-        da_b_std = da_b.std(dim=dim)
+        # Compute std and corr
+        if weights is not None:
+            da_a_std = da_a.weighted(weights).std(dim=dim)
+            da_b_std = da_b.weighted(weights).std(dim=dim)
+        else:
+            da_a_std = da_a.std(dim=dim)
+            da_b_std = da_b.std(dim=dim)
         corr = cov / (da_a_std * da_b_std)
-        return corr  # type: ignore[return-value]
+        return cast(T_DataArray, corr)
 
 
 def cross(
@@ -1442,7 +1585,7 @@ def cross(
     >>> a = xr.DataArray([1, 2, 3])
     >>> b = xr.DataArray([4, 5, 6])
     >>> xr.cross(a, b, dim="dim_0")
-    <xarray.DataArray (dim_0: 3)>
+    <xarray.DataArray (dim_0: 3)> Size: 24B
     array([-3,  6, -3])
     Dimensions without coordinates: dim_0
 
@@ -1452,7 +1595,7 @@ def cross(
     >>> a = xr.DataArray([1, 2])
     >>> b = xr.DataArray([4, 5])
     >>> xr.cross(a, b, dim="dim_0")
-    <xarray.DataArray ()>
+    <xarray.DataArray ()> Size: 8B
     array(-3)
 
     Vector cross-product with 3 dimensions but zeros at the last axis
@@ -1461,7 +1604,7 @@ def cross(
     >>> a = xr.DataArray([1, 2, 0])
     >>> b = xr.DataArray([4, 5, 0])
     >>> xr.cross(a, b, dim="dim_0")
-    <xarray.DataArray (dim_0: 3)>
+    <xarray.DataArray (dim_0: 3)> Size: 24B
     array([ 0,  0, -3])
     Dimensions without coordinates: dim_0
 
@@ -1478,10 +1621,10 @@ def cross(
     ...     coords=dict(cartesian=(["cartesian"], ["x", "y", "z"])),
     ... )
     >>> xr.cross(a, b, dim="cartesian")
-    <xarray.DataArray (cartesian: 3)>
+    <xarray.DataArray (cartesian: 3)> Size: 24B
     array([12, -6, -3])
     Coordinates:
-      * cartesian  (cartesian) <U1 'x' 'y' 'z'
+      * cartesian  (cartesian) <U1 12B 'x' 'y' 'z'
 
     One vector with dimension 2 but coords in other positions:
 
@@ -1496,10 +1639,10 @@ def cross(
     ...     coords=dict(cartesian=(["cartesian"], ["x", "y", "z"])),
     ... )
     >>> xr.cross(a, b, dim="cartesian")
-    <xarray.DataArray (cartesian: 3)>
+    <xarray.DataArray (cartesian: 3)> Size: 24B
     array([-10,   2,   5])
     Coordinates:
-      * cartesian  (cartesian) <U1 'x' 'y' 'z'
+      * cartesian  (cartesian) <U1 12B 'x' 'y' 'z'
 
     Multiple vector cross-products. Note that the direction of the
     cross product vector is defined by the right-hand rule:
@@ -1521,12 +1664,12 @@ def cross(
     ...     ),
     ... )
     >>> xr.cross(a, b, dim="cartesian")
-    <xarray.DataArray (time: 2, cartesian: 3)>
+    <xarray.DataArray (time: 2, cartesian: 3)> Size: 48B
     array([[-3,  6, -3],
            [ 3, -6,  3]])
     Coordinates:
-      * time       (time) int64 0 1
-      * cartesian  (cartesian) <U1 'x' 'y' 'z'
+      * time       (time) int64 16B 0 1
+      * cartesian  (cartesian) <U1 12B 'x' 'y' 'z'
 
     Cross can be called on Datasets by converting to DataArrays and later
     back to a Dataset:
@@ -1534,16 +1677,18 @@ def cross(
     >>> ds_a = xr.Dataset(dict(x=("dim_0", [1]), y=("dim_0", [2]), z=("dim_0", [3])))
     >>> ds_b = xr.Dataset(dict(x=("dim_0", [4]), y=("dim_0", [5]), z=("dim_0", [6])))
     >>> c = xr.cross(
-    ...     ds_a.to_array("cartesian"), ds_b.to_array("cartesian"), dim="cartesian"
+    ...     ds_a.to_dataarray("cartesian"),
+    ...     ds_b.to_dataarray("cartesian"),
+    ...     dim="cartesian",
     ... )
     >>> c.to_dataset(dim="cartesian")
-    <xarray.Dataset>
+    <xarray.Dataset> Size: 24B
     Dimensions:  (dim_0: 1)
     Dimensions without coordinates: dim_0
     Data variables:
-        x        (dim_0) int64 -3
-        y        (dim_0) int64 6
-        z        (dim_0) int64 -3
+        x        (dim_0) int64 8B -3
+        y        (dim_0) int64 8B 6
+        z        (dim_0) int64 8B -3
 
     See Also
     --------
@@ -1620,29 +1765,41 @@ def cross(
     return c
 
 
+@deprecate_dims
 def dot(
     *arrays,
-    dims: Dims = None,
+    dim: Dims = None,
     **kwargs: Any,
 ):
-    """Generalized dot product for xarray objects. Like np.einsum, but
-    provides a simpler interface based on array dimensions.
+    """Generalized dot product for xarray objects. Like ``np.einsum``, but
+    provides a simpler interface based on array dimension names.
 
     Parameters
     ----------
     *arrays : DataArray or Variable
         Arrays to compute.
-    dims : str, iterable of hashable, "..." or None, optional
+    dim : str, iterable of hashable, "..." or None, optional
         Which dimensions to sum over. Ellipsis ('...') sums over all dimensions.
         If not specified, then all the common dimensions are summed over.
     **kwargs : dict
-        Additional keyword arguments passed to numpy.einsum or
-        dask.array.einsum
+        Additional keyword arguments passed to ``numpy.einsum`` or
+        ``dask.array.einsum``
 
     Returns
     -------
     DataArray
 
+    See Also
+    --------
+    numpy.einsum
+    dask.array.einsum
+    opt_einsum.contract
+
+    Notes
+    -----
+    We recommend installing the optional ``opt_einsum`` package, or alternatively passing ``optimize=True``,
+    which is passed through to ``np.einsum``, and works for most array backends.
+
     Examples
     --------
     >>> da_a = xr.DataArray(np.arange(3 * 2).reshape(3, 2), dims=["a", "b"])
@@ -1650,14 +1807,14 @@ def dot(
     >>> da_c = xr.DataArray(np.arange(2 * 3).reshape(2, 3), dims=["c", "d"])
 
     >>> da_a
-    <xarray.DataArray (a: 3, b: 2)>
+    <xarray.DataArray (a: 3, b: 2)> Size: 48B
     array([[0, 1],
            [2, 3],
            [4, 5]])
     Dimensions without coordinates: a, b
 
     >>> da_b
-    <xarray.DataArray (a: 3, b: 2, c: 2)>
+    <xarray.DataArray (a: 3, b: 2, c: 2)> Size: 96B
     array([[[ 0,  1],
             [ 2,  3]],
     <BLANKLINE>
@@ -1669,36 +1826,36 @@ def dot(
     Dimensions without coordinates: a, b, c
 
     >>> da_c
-    <xarray.DataArray (c: 2, d: 3)>
+    <xarray.DataArray (c: 2, d: 3)> Size: 48B
     array([[0, 1, 2],
            [3, 4, 5]])
     Dimensions without coordinates: c, d
 
-    >>> xr.dot(da_a, da_b, dims=["a", "b"])
-    <xarray.DataArray (c: 2)>
+    >>> xr.dot(da_a, da_b, dim=["a", "b"])
+    <xarray.DataArray (c: 2)> Size: 16B
     array([110, 125])
     Dimensions without coordinates: c
 
-    >>> xr.dot(da_a, da_b, dims=["a"])
-    <xarray.DataArray (b: 2, c: 2)>
+    >>> xr.dot(da_a, da_b, dim=["a"])
+    <xarray.DataArray (b: 2, c: 2)> Size: 32B
     array([[40, 46],
            [70, 79]])
     Dimensions without coordinates: b, c
 
-    >>> xr.dot(da_a, da_b, da_c, dims=["b", "c"])
-    <xarray.DataArray (a: 3, d: 3)>
+    >>> xr.dot(da_a, da_b, da_c, dim=["b", "c"])
+    <xarray.DataArray (a: 3, d: 3)> Size: 72B
     array([[  9,  14,  19],
            [ 93, 150, 207],
            [273, 446, 619]])
     Dimensions without coordinates: a, d
 
     >>> xr.dot(da_a, da_b)
-    <xarray.DataArray (c: 2)>
+    <xarray.DataArray (c: 2)> Size: 16B
     array([110, 125])
     Dimensions without coordinates: c
 
-    >>> xr.dot(da_a, da_b, dims=...)
-    <xarray.DataArray ()>
+    >>> xr.dot(da_a, da_b, dim=...)
+    <xarray.DataArray ()> Size: 8B
     array(235)
     """
     from xarray.core.dataarray import DataArray
@@ -1707,7 +1864,7 @@ def dot(
     if any(not isinstance(arr, (Variable, DataArray)) for arr in arrays):
         raise TypeError(
             "Only xr.DataArray and xr.Variable are supported."
-            "Given {}.".format([type(arr) for arr in arrays])
+            f"Given {[type(arr) for arr in arrays]}."
         )
 
     if len(arrays) == 0:
@@ -1721,18 +1878,16 @@ def dot(
     einsum_axes = "abcdefghijklmnopqrstuvwxyz"
     dim_map = {d: einsum_axes[i] for i, d in enumerate(all_dims)}
 
-    if dims is ...:
-        dims = all_dims
-    elif isinstance(dims, str):
-        dims = (dims,)
-    elif dims is None:
-        # find dimensions that occur more than one times
+    if dim is None:
+        # find dimensions that occur more than once
         dim_counts: Counter = Counter()
         for arr in arrays:
             dim_counts.update(arr.dims)
-        dims = tuple(d for d, c in dim_counts.items() if c > 1)
+        dim = tuple(d for d, c in dim_counts.items() if c > 1)
+    else:
+        dim = parse_dims(dim, all_dims=tuple(all_dims))
 
-    dot_dims: set[Hashable] = set(dims)
+    dot_dims: set[Hashable] = set(dim)
 
     # dimensions to be parallelized
     broadcast_dims = common_dims - dot_dims
@@ -1804,16 +1959,16 @@ def where(cond, x, y, keep_attrs=None):
     ...     name="sst",
     ... )
     >>> x
-    <xarray.DataArray 'sst' (lat: 10)>
+    <xarray.DataArray 'sst' (lat: 10)> Size: 80B
     array([0. , 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9])
     Coordinates:
-      * lat      (lat) int64 0 1 2 3 4 5 6 7 8 9
+      * lat      (lat) int64 80B 0 1 2 3 4 5 6 7 8 9
 
     >>> xr.where(x < 0.5, x, x * 100)
-    <xarray.DataArray 'sst' (lat: 10)>
+    <xarray.DataArray 'sst' (lat: 10)> Size: 80B
     array([ 0. ,  0.1,  0.2,  0.3,  0.4, 50. , 60. , 70. , 80. , 90. ])
     Coordinates:
-      * lat      (lat) int64 0 1 2 3 4 5 6 7 8 9
+      * lat      (lat) int64 80B 0 1 2 3 4 5 6 7 8 9
 
     >>> y = xr.DataArray(
     ...     0.1 * np.arange(9).reshape(3, 3),
@@ -1822,27 +1977,27 @@ def where(cond, x, y, keep_attrs=None):
     ...     name="sst",
     ... )
     >>> y
-    <xarray.DataArray 'sst' (lat: 3, lon: 3)>
+    <xarray.DataArray 'sst' (lat: 3, lon: 3)> Size: 72B
     array([[0. , 0.1, 0.2],
            [0.3, 0.4, 0.5],
            [0.6, 0.7, 0.8]])
     Coordinates:
-      * lat      (lat) int64 0 1 2
-      * lon      (lon) int64 10 11 12
+      * lat      (lat) int64 24B 0 1 2
+      * lon      (lon) int64 24B 10 11 12
 
     >>> xr.where(y.lat < 1, y, -1)
-    <xarray.DataArray (lat: 3, lon: 3)>
+    <xarray.DataArray (lat: 3, lon: 3)> Size: 72B
     array([[ 0. ,  0.1,  0.2],
            [-1. , -1. , -1. ],
            [-1. , -1. , -1. ]])
     Coordinates:
-      * lat      (lat) int64 0 1 2
-      * lon      (lon) int64 10 11 12
+      * lat      (lat) int64 24B 0 1 2
+      * lon      (lon) int64 24B 10 11 12
 
     >>> cond = xr.DataArray([True, False], dims=["x"])
     >>> x = xr.DataArray([1, 2], dims=["y"])
     >>> xr.where(cond, x, 0)
-    <xarray.DataArray (x: 2, y: 2)>
+    <xarray.DataArray (x: 2, y: 2)> Size: 32B
     array([[1, 2],
            [0, 0]])
     Dimensions without coordinates: x, y
@@ -1895,29 +2050,25 @@ def where(cond, x, y, keep_attrs=None):
 @overload
 def polyval(
     coord: DataArray, coeffs: DataArray, degree_dim: Hashable = "degree"
-) -> DataArray:
-    ...
+) -> DataArray: ...
 
 
 @overload
 def polyval(
     coord: DataArray, coeffs: Dataset, degree_dim: Hashable = "degree"
-) -> Dataset:
-    ...
+) -> Dataset: ...
 
 
 @overload
 def polyval(
     coord: Dataset, coeffs: DataArray, degree_dim: Hashable = "degree"
-) -> Dataset:
-    ...
+) -> Dataset: ...
 
 
 @overload
 def polyval(
     coord: Dataset, coeffs: Dataset, degree_dim: Hashable = "degree"
-) -> Dataset:
-    ...
+) -> Dataset: ...
 
 
 @overload
@@ -1925,8 +2076,7 @@ def polyval(
     coord: Dataset | DataArray,
     coeffs: Dataset | DataArray,
     degree_dim: Hashable = "degree",
-) -> Dataset | DataArray:
-    ...
+) -> Dataset | DataArray: ...
 
 
 def polyval(
@@ -2046,9 +2196,13 @@ def _calc_idxminmax(
         raise ValueError("Must supply 'dim' argument for multidimensional arrays")
 
     if dim not in array.dims:
-        raise KeyError(f'Dimension "{dim}" not in dimension')
+        raise KeyError(
+            f"Dimension {dim!r} not found in array dimensions {array.dims!r}"
+        )
     if dim not in array.coords:
-        raise KeyError(f'Dimension "{dim}" does not have coordinates')
+        raise KeyError(
+            f"Dimension {dim!r} is not one of the coordinates {tuple(array.coords.keys())}"
+        )
 
     # These are dtypes with NaN values argmin and argmax can handle
     na_dtypes = "cfO"
@@ -2066,7 +2220,8 @@ def _calc_idxminmax(
         chunkmanager = get_chunked_array_type(array.data)
         chunks = dict(zip(array.dims, array.chunks))
         dask_coord = chunkmanager.from_array(array[dim].data, chunks=chunks[dim])
-        res = indx.copy(data=dask_coord[indx.data.ravel()].reshape(indx.shape))
+        data = dask_coord[duck_array_ops.ravel(indx.data)]
+        res = indx.copy(data=duck_array_ops.reshape(data, indx.shape))
         # we need to attach back the dim name
         res.name = dim
     else:
@@ -2090,23 +2245,19 @@ _V = TypeVar("_V", bound=Union["Dataset", "DataArray"])
 
 
 @overload
-def unify_chunks(__obj: _T) -> tuple[_T]:
-    ...
+def unify_chunks(__obj: _T) -> tuple[_T]: ...
 
 
 @overload
-def unify_chunks(__obj1: _T, __obj2: _U) -> tuple[_T, _U]:
-    ...
+def unify_chunks(__obj1: _T, __obj2: _U) -> tuple[_T, _U]: ...
 
 
 @overload
-def unify_chunks(__obj1: _T, __obj2: _U, __obj3: _V) -> tuple[_T, _U, _V]:
-    ...
+def unify_chunks(__obj1: _T, __obj2: _U, __obj3: _V) -> tuple[_T, _U, _V]: ...
 
 
 @overload
-def unify_chunks(*objects: Dataset | DataArray) -> tuple[Dataset | DataArray, ...]:
-    ...
+def unify_chunks(*objects: Dataset | DataArray) -> tuple[Dataset | DataArray, ...]: ...
 
 
 def unify_chunks(*objects: Dataset | DataArray) -> tuple[Dataset | DataArray, ...]:
