@@ -1,7 +1,7 @@
 from __future__ import annotations
 
 from collections.abc import Hashable, Iterable
-from typing import TYPE_CHECKING, Any, Union, cast, overload
+from typing import TYPE_CHECKING, Any, Union, overload
 
 import numpy as np
 import pandas as pd
@@ -16,7 +16,7 @@ from xarray.core.merge import (
     merge_attrs,
     merge_collected,
 )
-from xarray.core.types import T_DataArray, T_Dataset
+from xarray.core.types import T_DataArray, T_Dataset, T_Variable
 from xarray.core.variable import Variable
 from xarray.core.variable import concat as concat_vars
 
@@ -34,7 +34,7 @@ if TYPE_CHECKING:
 @overload
 def concat(
     objs: Iterable[T_Dataset],
-    dim: Hashable | T_DataArray | pd.Index,
+    dim: Hashable | T_Variable | T_DataArray | pd.Index,
     data_vars: T_DataVars = "all",
     coords: ConcatOptions | list[Hashable] = "different",
     compat: CompatOptions = "equals",
@@ -42,14 +42,13 @@ def concat(
     fill_value: object = dtypes.NA,
     join: JoinOptions = "outer",
     combine_attrs: CombineAttrsOptions = "override",
-) -> T_Dataset:
-    ...
+) -> T_Dataset: ...
 
 
 @overload
 def concat(
     objs: Iterable[T_DataArray],
-    dim: Hashable | T_DataArray | pd.Index,
+    dim: Hashable | T_Variable | T_DataArray | pd.Index,
     data_vars: T_DataVars = "all",
     coords: ConcatOptions | list[Hashable] = "different",
     compat: CompatOptions = "equals",
@@ -57,8 +56,7 @@ def concat(
     fill_value: object = dtypes.NA,
     join: JoinOptions = "outer",
     combine_attrs: CombineAttrsOptions = "override",
-) -> T_DataArray:
-    ...
+) -> T_DataArray: ...
 
 
 def concat(
@@ -80,11 +78,11 @@ def concat(
         xarray objects to concatenate together. Each object is expected to
         consist of variables and coordinates with matching shapes except for
         along the concatenated dimension.
-    dim : Hashable or DataArray or pandas.Index
+    dim : Hashable or Variable or DataArray or pandas.Index
         Name of the dimension to concatenate along. This can either be a new
         dimension name, in which case it is added along axis=0, or an existing
         dimension name, in which case the location of the dimension is
-        unchanged. If dimension is provided as a DataArray or Index, its name
+        unchanged. If dimension is provided as a Variable, DataArray or Index, its name
         is used as the dimension to concatenate along and the values are added
         as a coordinate.
     data_vars : {"minimal", "different", "all"} or list of Hashable, optional
@@ -179,46 +177,46 @@ def concat(
     ...     np.arange(6).reshape(2, 3), [("x", ["a", "b"]), ("y", [10, 20, 30])]
     ... )
     >>> da
-    <xarray.DataArray (x: 2, y: 3)>
+    <xarray.DataArray (x: 2, y: 3)> Size: 48B
     array([[0, 1, 2],
            [3, 4, 5]])
     Coordinates:
-      * x        (x) <U1 'a' 'b'
-      * y        (y) int64 10 20 30
+      * x        (x) <U1 8B 'a' 'b'
+      * y        (y) int64 24B 10 20 30
 
     >>> xr.concat([da.isel(y=slice(0, 1)), da.isel(y=slice(1, None))], dim="y")
-    <xarray.DataArray (x: 2, y: 3)>
+    <xarray.DataArray (x: 2, y: 3)> Size: 48B
     array([[0, 1, 2],
            [3, 4, 5]])
     Coordinates:
-      * x        (x) <U1 'a' 'b'
-      * y        (y) int64 10 20 30
+      * x        (x) <U1 8B 'a' 'b'
+      * y        (y) int64 24B 10 20 30
 
     >>> xr.concat([da.isel(x=0), da.isel(x=1)], "x")
-    <xarray.DataArray (x: 2, y: 3)>
+    <xarray.DataArray (x: 2, y: 3)> Size: 48B
     array([[0, 1, 2],
            [3, 4, 5]])
     Coordinates:
-      * x        (x) <U1 'a' 'b'
-      * y        (y) int64 10 20 30
+      * x        (x) <U1 8B 'a' 'b'
+      * y        (y) int64 24B 10 20 30
 
     >>> xr.concat([da.isel(x=0), da.isel(x=1)], "new_dim")
-    <xarray.DataArray (new_dim: 2, y: 3)>
+    <xarray.DataArray (new_dim: 2, y: 3)> Size: 48B
     array([[0, 1, 2],
            [3, 4, 5]])
     Coordinates:
-        x        (new_dim) <U1 'a' 'b'
-      * y        (y) int64 10 20 30
+        x        (new_dim) <U1 8B 'a' 'b'
+      * y        (y) int64 24B 10 20 30
     Dimensions without coordinates: new_dim
 
     >>> xr.concat([da.isel(x=0), da.isel(x=1)], pd.Index([-90, -100], name="new_dim"))
-    <xarray.DataArray (new_dim: 2, y: 3)>
+    <xarray.DataArray (new_dim: 2, y: 3)> Size: 48B
     array([[0, 1, 2],
            [3, 4, 5]])
     Coordinates:
-        x        (new_dim) <U1 'a' 'b'
-      * y        (y) int64 10 20 30
-      * new_dim  (new_dim) int64 -90 -100
+        x        (new_dim) <U1 8B 'a' 'b'
+      * y        (y) int64 24B 10 20 30
+      * new_dim  (new_dim) int64 16B -90 -100
     """
     # TODO: add ignore_index arguments copied from pandas.concat
     # TODO: support concatenating scalar coordinates even if the concatenated
@@ -315,7 +313,7 @@ def _calc_concat_over(datasets, dim, dim_names, data_vars: T_DataVars, coords, c
                 if dim in ds:
                     ds = ds.set_coords(dim)
         concat_over.update(k for k, v in ds.variables.items() if dim in v.dims)
-        concat_dim_lengths.append(ds.dims.get(dim, 1))
+        concat_dim_lengths.append(ds.sizes.get(dim, 1))
 
     def process_subset_opt(opt, subset):
         if isinstance(opt, str):
@@ -391,17 +389,20 @@ def _calc_concat_over(datasets, dim, dim_names, data_vars: T_DataVars, coords, c
             else:
                 raise ValueError(f"unexpected value for {subset}: {opt}")
         else:
-            invalid_vars = [k for k in opt if k not in getattr(datasets[0], subset)]
+            valid_vars = tuple(getattr(datasets[0], subset))
+            invalid_vars = [k for k in opt if k not in valid_vars]
             if invalid_vars:
                 if subset == "coords":
                     raise ValueError(
-                        "some variables in coords are not coordinates on "
-                        f"the first dataset: {invalid_vars}"
+                        f"the variables {invalid_vars} in coords are not "
+                        f"found in the coordinates of the first dataset {valid_vars}"
                     )
                 else:
+                    # note: data_vars are not listed in the error message here,
+                    # because there may be lots of them
                     raise ValueError(
-                        "some variables in data_vars are not data variables "
-                        f"on the first dataset: {invalid_vars}"
+                        f"the variables {invalid_vars} in data_vars are not "
+                        f"found in the data variables of the first dataset"
                     )
             concat_over.update(opt)
 
@@ -428,7 +429,7 @@ def _parse_datasets(
     variables_order: dict[Hashable, Variable] = {}  # variables in order of appearance
 
     for ds in datasets:
-        dims_sizes.update(ds.dims)
+        dims_sizes.update(ds.sizes)
         all_coord_names.update(ds.coords)
         data_vars.update(ds.data_vars)
         variables_order.update(ds.variables)
@@ -447,7 +448,7 @@ def _parse_datasets(
 
 def _dataset_concat(
     datasets: list[T_Dataset],
-    dim: str | T_DataArray | pd.Index,
+    dim: str | T_Variable | T_DataArray | pd.Index,
     data_vars: T_DataVars,
     coords: str | list[str],
     compat: CompatOptions,
@@ -501,8 +502,7 @@ def _dataset_concat(
 
     # case where concat dimension is a coordinate or data_var but not a dimension
     if (dim in coord_names or dim in data_names) and dim not in dim_names:
-        # TODO: Overriding type because .expand_dims has incorrect typing:
-        datasets = [cast(T_Dataset, ds.expand_dims(dim)) for ds in datasets]
+        datasets = [ds.expand_dims(dim) for ds in datasets]
 
     # determine which variables to concatenate
     concat_over, equals, concat_dim_lengths = _calc_concat_over(
@@ -534,9 +534,10 @@ def _dataset_concat(
     result_encoding = datasets[0].encoding
 
     # check that global attributes are fixed across all datasets if necessary
-    for ds in datasets[1:]:
-        if compat == "identical" and not utils.dict_equiv(ds.attrs, result_attrs):
-            raise ValueError("Dataset global attributes not equal.")
+    if compat == "identical":
+        for ds in datasets[1:]:
+            if not utils.dict_equiv(ds.attrs, result_attrs):
+                raise ValueError("Dataset global attributes not equal.")
 
     # we've already verified everything is consistent; now, calculate
     # shared dimension sizes so we can expand the necessary variables
@@ -674,7 +675,7 @@ def _dataset_concat(
 
 def _dataarray_concat(
     arrays: Iterable[T_DataArray],
-    dim: str | T_DataArray | pd.Index,
+    dim: str | T_Variable | T_DataArray | pd.Index,
     data_vars: T_DataVars,
     coords: str | list[str],
     compat: CompatOptions,
@@ -705,8 +706,7 @@ def _dataarray_concat(
             if compat == "identical":
                 raise ValueError("array names not identical")
             else:
-                # TODO: Overriding type because .rename has incorrect typing:
-                arr = cast(T_DataArray, arr.rename(name))
+                arr = arr.rename(name)
         datasets.append(arr._to_temp_dataset())
 
     ds = _dataset_concat(
