@@ -4,7 +4,7 @@ import enum
 import functools
 import operator
 from collections import Counter, defaultdict
-from collections.abc import Hashable, Iterable, Mapping
+from collections.abc import Hashable, Mapping
 from contextlib import suppress
 from dataclasses import dataclass, field
 from datetime import timedelta
@@ -17,21 +17,18 @@ import pandas as pd
 from xarray.core import duck_array_ops
 from xarray.core.nputils import NumpyVIndexAdapter
 from xarray.core.options import OPTIONS
-from xarray.core.parallelcompat import get_chunked_array_type, is_chunked_array
-from xarray.core.pycompat import (
-    array_type,
-    integer_types,
-    is_duck_array,
-    is_duck_dask_array,
-)
 from xarray.core.types import T_Xarray
 from xarray.core.utils import (
     NDArrayMixin,
     either_dict_or_kwargs,
     get_valid_numpy_dtype,
+    is_duck_array,
+    is_duck_dask_array,
     is_scalar,
     to_0d_array,
 )
+from xarray.namedarray.parallelcompat import get_chunked_array_type
+from xarray.namedarray.pycompat import array_type, integer_types, is_chunked_array
 
 if TYPE_CHECKING:
     from numpy.typing import DTypeLike
@@ -143,7 +140,10 @@ def group_indexers_by_index(
         elif key in obj.coords:
             raise KeyError(f"no index found for coordinate {key!r}")
         elif key not in obj.dims:
-            raise KeyError(f"{key!r} is not a valid dimension or coordinate")
+            raise KeyError(
+                f"{key!r} is not a valid dimension or coordinate for "
+                f"{obj.__class__.__name__} with dimensions {obj.dims!r}"
+            )
         elif len(options):
             raise ValueError(
                 f"cannot supply selection options {options!r} for dimension {key!r}"
@@ -1057,9 +1057,11 @@ def _decompose_outer_indexer(
         # some backends such as h5py supports only 1 vector in indexers
         # We choose the most efficient axis
         gains = [
-            (np.max(k) - np.min(k) + 1.0) / len(np.unique(k))
-            if isinstance(k, np.ndarray)
-            else 0
+            (
+                (np.max(k) - np.min(k) + 1.0) / len(np.unique(k))
+                if isinstance(k, np.ndarray)
+                else 0
+            )
             for k in indexer_elems
         ]
         array_index = np.argmax(np.array(gains)) if len(gains) > 0 else None
@@ -1272,9 +1274,11 @@ def posify_mask_indexer(indexer):
         replaced by an adjacent non-masked element.
     """
     key = tuple(
-        _posify_mask_subindexer(k.ravel()).reshape(k.shape)
-        if isinstance(k, np.ndarray)
-        else k
+        (
+            _posify_mask_subindexer(k.ravel()).reshape(k.shape)
+            if isinstance(k, np.ndarray)
+            else k
+        )
         for k in indexer.tuple
     )
     return type(indexer)(key)
@@ -1303,7 +1307,7 @@ class NumpyIndexingAdapter(ExplicitlyIndexedNDArrayMixin):
         if not isinstance(array, np.ndarray):
             raise TypeError(
                 "NumpyIndexingAdapter only wraps np.ndarray. "
-                "Trying to wrap {}".format(type(array))
+                f"Trying to wrap {type(array)}"
             )
         self.array = array
 
@@ -1414,23 +1418,6 @@ class DaskIndexingAdapter(ExplicitlyIndexedNDArrayMixin):
         self.array = array
 
     def __getitem__(self, key):
-        if not isinstance(key, VectorizedIndexer):
-            # if possible, short-circuit when keys are effectively slice(None)
-            # This preserves dask name and passes lazy array equivalence checks
-            # (see duck_array_ops.lazy_array_equiv)
-            rewritten_indexer = False
-            new_indexer = []
-            for idim, k in enumerate(key.tuple):
-                if isinstance(k, Iterable) and (
-                    not is_duck_dask_array(k)
-                    and duck_array_ops.array_equiv(k, np.arange(self.array.shape[idim]))
-                ):
-                    new_indexer.append(slice(None))
-                    rewritten_indexer = True
-                else:
-                    new_indexer.append(k)
-            if rewritten_indexer:
-                key = type(key)(tuple(new_indexer))
 
         if isinstance(key, BasicIndexer):
             return self.array[key.tuple]
