@@ -474,10 +474,11 @@ def coerce_pandas_values(objects: Iterable[CoercibleMapping]) -> list[DatasetLik
     from xarray.core.dataarray import DataArray
     from xarray.core.dataset import Dataset
 
-    out = []
+    out: list[DatasetLike] = []
     for obj in objects:
+        variables: DatasetLike
         if isinstance(obj, (Dataset, Coordinates)):
-            variables: DatasetLike = obj
+            variables = obj
         else:
             variables = {}
             if isinstance(obj, PANDAS_TYPES):
@@ -491,7 +492,7 @@ def coerce_pandas_values(objects: Iterable[CoercibleMapping]) -> list[DatasetLik
 
 
 def _get_priority_vars_and_indexes(
-    objects: list[DatasetLike],
+    objects: Sequence[DatasetLike],
     priority_arg: int | None,
     compat: CompatOptions = "equals",
 ) -> dict[Hashable, MergeElement]:
@@ -503,7 +504,7 @@ def _get_priority_vars_and_indexes(
 
     Parameters
     ----------
-    objects : list of dict-like of Variable
+    objects : sequence of dict-like of Variable
         Dictionaries in which to find the priority variables.
     priority_arg : int or None
         Integer object whose variable should take priority.
@@ -723,6 +724,9 @@ def merge_core(
     dims = calculate_dimensions(variables)
 
     coord_names, noncoord_names = determine_coords(coerced)
+    if compat == "minimal":
+        # coordinates may be dropped in merged results
+        coord_names.intersection_update(variables)
     if explicit_coords is not None:
         assert_valid_explicit_coords(variables, dims, explicit_coords)
         coord_names.update(explicit_coords)
@@ -835,124 +839,124 @@ def merge(
     ... )
 
     >>> x
-    <xarray.DataArray 'var1' (lat: 2, lon: 2)>
+    <xarray.DataArray 'var1' (lat: 2, lon: 2)> Size: 32B
     array([[1., 2.],
            [3., 5.]])
     Coordinates:
-      * lat      (lat) float64 35.0 40.0
-      * lon      (lon) float64 100.0 120.0
+      * lat      (lat) float64 16B 35.0 40.0
+      * lon      (lon) float64 16B 100.0 120.0
 
     >>> y
-    <xarray.DataArray 'var2' (lat: 2, lon: 2)>
+    <xarray.DataArray 'var2' (lat: 2, lon: 2)> Size: 32B
     array([[5., 6.],
            [7., 8.]])
     Coordinates:
-      * lat      (lat) float64 35.0 42.0
-      * lon      (lon) float64 100.0 150.0
+      * lat      (lat) float64 16B 35.0 42.0
+      * lon      (lon) float64 16B 100.0 150.0
 
     >>> z
-    <xarray.DataArray 'var3' (time: 2, lon: 2)>
+    <xarray.DataArray 'var3' (time: 2, lon: 2)> Size: 32B
     array([[0., 3.],
            [4., 9.]])
     Coordinates:
-      * time     (time) float64 30.0 60.0
-      * lon      (lon) float64 100.0 150.0
+      * time     (time) float64 16B 30.0 60.0
+      * lon      (lon) float64 16B 100.0 150.0
 
     >>> xr.merge([x, y, z])
-    <xarray.Dataset>
+    <xarray.Dataset> Size: 256B
     Dimensions:  (lat: 3, lon: 3, time: 2)
     Coordinates:
-      * lat      (lat) float64 35.0 40.0 42.0
-      * lon      (lon) float64 100.0 120.0 150.0
-      * time     (time) float64 30.0 60.0
+      * lat      (lat) float64 24B 35.0 40.0 42.0
+      * lon      (lon) float64 24B 100.0 120.0 150.0
+      * time     (time) float64 16B 30.0 60.0
     Data variables:
-        var1     (lat, lon) float64 1.0 2.0 nan 3.0 5.0 nan nan nan nan
-        var2     (lat, lon) float64 5.0 nan 6.0 nan nan nan 7.0 nan 8.0
-        var3     (time, lon) float64 0.0 nan 3.0 4.0 nan 9.0
+        var1     (lat, lon) float64 72B 1.0 2.0 nan 3.0 5.0 nan nan nan nan
+        var2     (lat, lon) float64 72B 5.0 nan 6.0 nan nan nan 7.0 nan 8.0
+        var3     (time, lon) float64 48B 0.0 nan 3.0 4.0 nan 9.0
 
     >>> xr.merge([x, y, z], compat="identical")
-    <xarray.Dataset>
+    <xarray.Dataset> Size: 256B
     Dimensions:  (lat: 3, lon: 3, time: 2)
     Coordinates:
-      * lat      (lat) float64 35.0 40.0 42.0
-      * lon      (lon) float64 100.0 120.0 150.0
-      * time     (time) float64 30.0 60.0
+      * lat      (lat) float64 24B 35.0 40.0 42.0
+      * lon      (lon) float64 24B 100.0 120.0 150.0
+      * time     (time) float64 16B 30.0 60.0
     Data variables:
-        var1     (lat, lon) float64 1.0 2.0 nan 3.0 5.0 nan nan nan nan
-        var2     (lat, lon) float64 5.0 nan 6.0 nan nan nan 7.0 nan 8.0
-        var3     (time, lon) float64 0.0 nan 3.0 4.0 nan 9.0
+        var1     (lat, lon) float64 72B 1.0 2.0 nan 3.0 5.0 nan nan nan nan
+        var2     (lat, lon) float64 72B 5.0 nan 6.0 nan nan nan 7.0 nan 8.0
+        var3     (time, lon) float64 48B 0.0 nan 3.0 4.0 nan 9.0
 
     >>> xr.merge([x, y, z], compat="equals")
-    <xarray.Dataset>
+    <xarray.Dataset> Size: 256B
     Dimensions:  (lat: 3, lon: 3, time: 2)
     Coordinates:
-      * lat      (lat) float64 35.0 40.0 42.0
-      * lon      (lon) float64 100.0 120.0 150.0
-      * time     (time) float64 30.0 60.0
+      * lat      (lat) float64 24B 35.0 40.0 42.0
+      * lon      (lon) float64 24B 100.0 120.0 150.0
+      * time     (time) float64 16B 30.0 60.0
     Data variables:
-        var1     (lat, lon) float64 1.0 2.0 nan 3.0 5.0 nan nan nan nan
-        var2     (lat, lon) float64 5.0 nan 6.0 nan nan nan 7.0 nan 8.0
-        var3     (time, lon) float64 0.0 nan 3.0 4.0 nan 9.0
+        var1     (lat, lon) float64 72B 1.0 2.0 nan 3.0 5.0 nan nan nan nan
+        var2     (lat, lon) float64 72B 5.0 nan 6.0 nan nan nan 7.0 nan 8.0
+        var3     (time, lon) float64 48B 0.0 nan 3.0 4.0 nan 9.0
 
     >>> xr.merge([x, y, z], compat="equals", fill_value=-999.0)
-    <xarray.Dataset>
+    <xarray.Dataset> Size: 256B
     Dimensions:  (lat: 3, lon: 3, time: 2)
     Coordinates:
-      * lat      (lat) float64 35.0 40.0 42.0
-      * lon      (lon) float64 100.0 120.0 150.0
-      * time     (time) float64 30.0 60.0
+      * lat      (lat) float64 24B 35.0 40.0 42.0
+      * lon      (lon) float64 24B 100.0 120.0 150.0
+      * time     (time) float64 16B 30.0 60.0
     Data variables:
-        var1     (lat, lon) float64 1.0 2.0 -999.0 3.0 ... -999.0 -999.0 -999.0
-        var2     (lat, lon) float64 5.0 -999.0 6.0 -999.0 ... -999.0 7.0 -999.0 8.0
-        var3     (time, lon) float64 0.0 -999.0 3.0 4.0 -999.0 9.0
+        var1     (lat, lon) float64 72B 1.0 2.0 -999.0 3.0 ... -999.0 -999.0 -999.0
+        var2     (lat, lon) float64 72B 5.0 -999.0 6.0 -999.0 ... 7.0 -999.0 8.0
+        var3     (time, lon) float64 48B 0.0 -999.0 3.0 4.0 -999.0 9.0
 
     >>> xr.merge([x, y, z], join="override")
-    <xarray.Dataset>
+    <xarray.Dataset> Size: 144B
     Dimensions:  (lat: 2, lon: 2, time: 2)
     Coordinates:
-      * lat      (lat) float64 35.0 40.0
-      * lon      (lon) float64 100.0 120.0
-      * time     (time) float64 30.0 60.0
+      * lat      (lat) float64 16B 35.0 40.0
+      * lon      (lon) float64 16B 100.0 120.0
+      * time     (time) float64 16B 30.0 60.0
     Data variables:
-        var1     (lat, lon) float64 1.0 2.0 3.0 5.0
-        var2     (lat, lon) float64 5.0 6.0 7.0 8.0
-        var3     (time, lon) float64 0.0 3.0 4.0 9.0
+        var1     (lat, lon) float64 32B 1.0 2.0 3.0 5.0
+        var2     (lat, lon) float64 32B 5.0 6.0 7.0 8.0
+        var3     (time, lon) float64 32B 0.0 3.0 4.0 9.0
 
     >>> xr.merge([x, y, z], join="inner")
-    <xarray.Dataset>
+    <xarray.Dataset> Size: 64B
     Dimensions:  (lat: 1, lon: 1, time: 2)
     Coordinates:
-      * lat      (lat) float64 35.0
-      * lon      (lon) float64 100.0
-      * time     (time) float64 30.0 60.0
+      * lat      (lat) float64 8B 35.0
+      * lon      (lon) float64 8B 100.0
+      * time     (time) float64 16B 30.0 60.0
     Data variables:
-        var1     (lat, lon) float64 1.0
-        var2     (lat, lon) float64 5.0
-        var3     (time, lon) float64 0.0 4.0
+        var1     (lat, lon) float64 8B 1.0
+        var2     (lat, lon) float64 8B 5.0
+        var3     (time, lon) float64 16B 0.0 4.0
 
     >>> xr.merge([x, y, z], compat="identical", join="inner")
-    <xarray.Dataset>
+    <xarray.Dataset> Size: 64B
     Dimensions:  (lat: 1, lon: 1, time: 2)
     Coordinates:
-      * lat      (lat) float64 35.0
-      * lon      (lon) float64 100.0
-      * time     (time) float64 30.0 60.0
+      * lat      (lat) float64 8B 35.0
+      * lon      (lon) float64 8B 100.0
+      * time     (time) float64 16B 30.0 60.0
     Data variables:
-        var1     (lat, lon) float64 1.0
-        var2     (lat, lon) float64 5.0
-        var3     (time, lon) float64 0.0 4.0
+        var1     (lat, lon) float64 8B 1.0
+        var2     (lat, lon) float64 8B 5.0
+        var3     (time, lon) float64 16B 0.0 4.0
 
     >>> xr.merge([x, y, z], compat="broadcast_equals", join="outer")
-    <xarray.Dataset>
+    <xarray.Dataset> Size: 256B
     Dimensions:  (lat: 3, lon: 3, time: 2)
     Coordinates:
-      * lat      (lat) float64 35.0 40.0 42.0
-      * lon      (lon) float64 100.0 120.0 150.0
-      * time     (time) float64 30.0 60.0
+      * lat      (lat) float64 24B 35.0 40.0 42.0
+      * lon      (lon) float64 24B 100.0 120.0 150.0
+      * time     (time) float64 16B 30.0 60.0
     Data variables:
-        var1     (lat, lon) float64 1.0 2.0 nan 3.0 5.0 nan nan nan nan
-        var2     (lat, lon) float64 5.0 nan 6.0 nan nan nan 7.0 nan 8.0
-        var3     (time, lon) float64 0.0 nan 3.0 4.0 nan 9.0
+        var1     (lat, lon) float64 72B 1.0 2.0 nan 3.0 5.0 nan nan nan nan
+        var2     (lat, lon) float64 72B 5.0 nan 6.0 nan nan nan 7.0 nan 8.0
+        var3     (time, lon) float64 48B 0.0 nan 3.0 4.0 nan 9.0
 
     >>> xr.merge([x, y, z], join="exact")
     Traceback (most recent call last):
