@@ -6,10 +6,8 @@ from typing import TYPE_CHECKING, Literal, TypedDict
 from xarray.core.utils import FrozenDict
 
 if TYPE_CHECKING:
-    try:
-        from matplotlib.colors import Colormap
-    except ImportError:
-        Colormap = str
+    from matplotlib.colors import Colormap
+
     Options = Literal[
         "arithmetic_join",
         "cmap_divergent",
@@ -22,6 +20,7 @@ if TYPE_CHECKING:
         "display_expand_coords",
         "display_expand_data_vars",
         "display_expand_data",
+        "display_expand_groups",
         "display_expand_indexes",
         "display_default_indexes",
         "enable_cftimeindex",
@@ -29,6 +28,8 @@ if TYPE_CHECKING:
         "keep_attrs",
         "warn_for_unclosed_files",
         "use_bottleneck",
+        "use_numbagg",
+        "use_opt_einsum",
         "use_flox",
     ]
 
@@ -44,6 +45,7 @@ if TYPE_CHECKING:
         display_expand_coords: Literal["default", True, False]
         display_expand_data_vars: Literal["default", True, False]
         display_expand_data: Literal["default", True, False]
+        display_expand_groups: Literal["default", True, False]
         display_expand_indexes: Literal["default", True, False]
         display_default_indexes: Literal["default", True, False]
         enable_cftimeindex: bool
@@ -52,6 +54,8 @@ if TYPE_CHECKING:
         warn_for_unclosed_files: bool
         use_bottleneck: bool
         use_flox: bool
+        use_numbagg: bool
+        use_opt_einsum: bool
 
 
 OPTIONS: T_Options = {
@@ -66,6 +70,7 @@ OPTIONS: T_Options = {
     "display_expand_coords": "default",
     "display_expand_data_vars": "default",
     "display_expand_data": "default",
+    "display_expand_groups": "default",
     "display_expand_indexes": "default",
     "display_default_indexes": False,
     "enable_cftimeindex": True,
@@ -74,6 +79,8 @@ OPTIONS: T_Options = {
     "warn_for_unclosed_files": False,
     "use_bottleneck": True,
     "use_flox": True,
+    "use_numbagg": True,
+    "use_opt_einsum": True,
 }
 
 _JOIN_OPTIONS = frozenset(["inner", "outer", "left", "right", "exact"])
@@ -100,6 +107,8 @@ _VALIDATORS = {
     "file_cache_maxsize": _positive_integer,
     "keep_attrs": lambda choice: choice in [True, False, "default"],
     "use_bottleneck": lambda value: isinstance(value, bool),
+    "use_numbagg": lambda value: isinstance(value, bool),
+    "use_opt_einsum": lambda value: isinstance(value, bool),
     "use_flox": lambda value: isinstance(value, bool),
     "warn_for_unclosed_files": lambda value: isinstance(value, bool),
 }
@@ -164,11 +173,11 @@ class set_options:
     cmap_divergent : str or matplotlib.colors.Colormap, default: "RdBu_r"
         Colormap to use for divergent data plots. If string, must be
         matplotlib built-in colormap. Can also be a Colormap object
-        (e.g. mpl.cm.magma)
+        (e.g. mpl.colormaps["magma"])
     cmap_sequential : str or matplotlib.colors.Colormap, default: "viridis"
         Colormap to use for nondivergent data plots. If string, must be
         matplotlib built-in colormap. Can also be a Colormap object
-        (e.g. mpl.cm.magma)
+        (e.g. mpl.colormaps["magma"])
     display_expand_attrs : {"default", True, False}
         Whether to expand the attributes section for display of
         ``DataArray`` or ``Dataset`` objects. Can be
@@ -232,6 +241,11 @@ class set_options:
     use_flox : bool, default: True
         Whether to use ``numpy_groupies`` and `flox`` to
         accelerate groupby and resampling reductions.
+    use_numbagg : bool, default: True
+        Whether to use ``numbagg`` to accelerate reductions.
+        Takes precedence over ``use_bottleneck`` when both are True.
+    use_opt_einsum : bool, default: True
+        Whether to use ``opt_einsum`` to accelerate dot products.
     warn_for_unclosed_files : bool, default: False
         Whether or not to issue a warning when unclosed files are
         deallocated. This is mostly useful for debugging.
@@ -244,10 +258,10 @@ class set_options:
     >>> with xr.set_options(display_width=40):
     ...     print(ds)
     ...
-    <xarray.Dataset>
+    <xarray.Dataset> Size: 8kB
     Dimensions:  (x: 1000)
     Coordinates:
-      * x        (x) int64 0 1 2 ... 998 999
+      * x        (x) int64 8kB 0 1 ... 999
     Data variables:
         *empty*
 
