@@ -1,4 +1,5 @@
 """Internal utilities; not for external use"""
+
 # Some functions in this module are derived from functions in pandas. For
 # reference, here is a copy of the pandas copyright notice:
 
@@ -37,7 +38,6 @@ from __future__ import annotations
 
 import contextlib
 import functools
-import importlib
 import inspect
 import io
 import itertools
@@ -50,14 +50,17 @@ from collections.abc import (
     Collection,
     Container,
     Hashable,
+    ItemsView,
     Iterable,
     Iterator,
+    KeysView,
     Mapping,
     MutableMapping,
     MutableSet,
-    Sequence,
+    ValuesView,
 )
 from enum import Enum
+from pathlib import Path
 from typing import (
     TYPE_CHECKING,
     Any,
@@ -65,15 +68,27 @@ from typing import (
     Generic,
     Literal,
     TypeVar,
-    cast,
     overload,
 )
 
 import numpy as np
 import pandas as pd
 
+from xarray.namedarray.utils import (  # noqa: F401
+    ReprObject,
+    drop_missing_dims,
+    either_dict_or_kwargs,
+    infix_dims,
+    is_dask_collection,
+    is_dict_like,
+    is_duck_array,
+    is_duck_dask_array,
+    module_available,
+    to_0d_object_array,
+)
+
 if TYPE_CHECKING:
-    from xarray.core.types import Dims, ErrorOptionsWithWarn, OrderedDims
+    from xarray.core.types import Dims, ErrorOptionsWithWarn
 
 K = TypeVar("K")
 V = TypeVar("V")
@@ -114,6 +129,8 @@ def get_valid_numpy_dtype(array: np.ndarray | pd.Index):
     elif hasattr(array, "categories"):
         # category isn't a real numpy dtype
         dtype = array.categories.dtype
+        if not is_valid_numpy_dtype(dtype):
+            dtype = np.dtype("O")
     elif not is_valid_numpy_dtype(array.dtype):
         dtype = np.dtype("O")
     else:
@@ -240,11 +257,6 @@ def remove_incompatible_items(
             del first_dict[k]
 
 
-# It's probably OK to give this as a TypeGuard; though it's not perfectly robust.
-def is_dict_like(value: Any) -> TypeGuard[Mapping]:
-    return hasattr(value, "keys") and hasattr(value, "__getitem__")
-
-
 def is_full_slice(value: Any) -> bool:
     return isinstance(value, slice) and value == slice(None)
 
@@ -253,39 +265,6 @@ def is_list_like(value: Any) -> TypeGuard[list | tuple]:
     return isinstance(value, (list, tuple))
 
 
-def is_duck_array(value: Any) -> bool:
-    if isinstance(value, np.ndarray):
-        return True
-    return (
-        hasattr(value, "ndim")
-        and hasattr(value, "shape")
-        and hasattr(value, "dtype")
-        and (
-            (hasattr(value, "__array_function__") and hasattr(value, "__array_ufunc__"))
-            or hasattr(value, "__array_namespace__")
-        )
-    )
-
-
-def either_dict_or_kwargs(
-    pos_kwargs: Mapping[Any, T] | None,
-    kw_kwargs: Mapping[str, T],
-    func_name: str,
-) -> Mapping[Hashable, T]:
-    if pos_kwargs is None or pos_kwargs == {}:
-        # Need an explicit cast to appease mypy due to invariance; see
-        # https://github.com/python/mypy/issues/6228
-        return cast(Mapping[Hashable, T], kw_kwargs)
-
-    if not is_dict_like(pos_kwargs):
-        raise ValueError(f"the first argument to .{func_name} must be a dictionary")
-    if kw_kwargs:
-        raise ValueError(
-            f"cannot specify both keyword and positional arguments to .{func_name}"
-        )
-    return pos_kwargs
-
-
 def _is_scalar(value, include_0d):
     from xarray.core.variable import NON_NUMPY_SUPPORTED_ARRAY_TYPES
 
@@ -341,13 +320,6 @@ def is_valid_numpy_dtype(dtype: Any) -> bool:
         return True
 
 
-def to_0d_object_array(value: Any) -> np.ndarray:
-    """Given a value, wrap it in a 0-D numpy.ndarray with dtype=object."""
-    result = np.empty((), dtype=object)
-    result[()] = value
-    return result
-
-
 def to_0d_array(value: Any) -> np.ndarray:
     """Given a value, wrap it in a 0-D numpy.ndarray."""
     if np.isscalar(value) or (isinstance(value, np.ndarray) and value.ndim == 0):
@@ -471,6 +443,55 @@ def FrozenDict(*args, **kwargs) -> Frozen:
     return Frozen(dict(*args, **kwargs))
 
 
+class FrozenMappingWarningOnValuesAccess(Frozen[K, V]):
+    """
+    Class which behaves like a Mapping but warns if the values are accessed.
+
+    Temporary object to aid in deprecation cycle of `Dataset.dims` (see GH issue #8496).
+    `Dataset.dims` is being changed from returning a mapping of dimension names to lengths to just
+    returning a frozen set of dimension names (to increase consistency with `DataArray.dims`).
+    This class retains backwards compatibility but raises a warning only if the return value
+    of ds.dims is used like a dictionary (i.e. it doesn't raise a warning if used in a way that
+    would also be valid for a FrozenSet, e.g. iteration).
+    """
+
+    __slots__ = ("mapping",)
+
+    def _warn(self) -> None:
+        emit_user_level_warning(
+            "The return type of `Dataset.dims` will be changed to return a set of dimension names in future, "
+            "in order to be more consistent with `DataArray.dims`. To access a mapping from dimension names to lengths, "
+            "please use `Dataset.sizes`.",
+            FutureWarning,
+        )
+
+    def __getitem__(self, key: K) -> V:
+        self._warn()
+        return super().__getitem__(key)
+
+    @overload
+    def get(self, key: K, /) -> V | None: ...
+
+    @overload
+    def get(self, key: K, /, default: V | T) -> V | T: ...
+
+    def get(self, key: K, default: T | None = None) -> V | T | None:
+        self._warn()
+        return super().get(key, default)
+
+    def keys(self) -> KeysView[K]:
+        self._warn()
+        return super().keys()
+
+    def items(self) -> ItemsView[K, V]:
+        self._warn()
+        return super().items()
+
+    def values(self) -> ValuesView[V]:
+        self._warn()
+        return super().values()
+
+
 class HybridMappingProxy(Mapping[K, V]):
     """Implements the Mapping interface. Uses the wrapped mapping for item lookup
     and a separate wrapped keys collection for iteration.
@@ -605,31 +626,6 @@ class NDArrayMixin(NdimSizeLenMixin):
         return f"{type(self).__name__}(array={self.array!r})"
 
 
-class ReprObject:
-    """Object that prints as the given value, for use with sentinel values."""
-
-    __slots__ = ("_value",)
-
-    def __init__(self, value: str):
-        self._value = value
-
-    def __repr__(self) -> str:
-        return self._value
-
-    def __eq__(self, other) -> bool:
-        if isinstance(other, ReprObject):
-            return self._value == other._value
-        return False
-
-    def __hash__(self) -> int:
-        return hash((type(self), self._value))
-
-    def __dask_tokenize__(self):
-        from dask.base import normalize_token
-
-        return normalize_token((type(self), self._value))
-
-
 @contextlib.contextmanager
 def close_on_error(f):
     """Context manager to ensure that a file opened by xarray is closed if an
@@ -789,36 +785,6 @@ class HiddenKeyDict(MutableMapping[K, V]):
         return len(self._data) - num_hidden
 
 
-def infix_dims(
-    dims_supplied: Collection,
-    dims_all: Collection,
-    missing_dims: ErrorOptionsWithWarn = "raise",
-) -> Iterator:
-    """
-    Resolves a supplied list containing an ellipsis representing other items, to
-    a generator with the 'realized' list of all items
-    """
-    if ... in dims_supplied:
-        if len(set(dims_all)) != len(dims_all):
-            raise ValueError("Cannot use ellipsis with repeated dims")
-        if list(dims_supplied).count(...) > 1:
-            raise ValueError("More than one ellipsis supplied")
-        other_dims = [d for d in dims_all if d not in dims_supplied]
-        existing_dims = drop_missing_dims(dims_supplied, dims_all, missing_dims)
-        for d in existing_dims:
-            if d is ...:
-                yield from other_dims
-            else:
-                yield d
-    else:
-        existing_dims = drop_missing_dims(dims_supplied, dims_all, missing_dims)
-        if set(existing_dims) ^ set(dims_all):
-            raise ValueError(
-                f"{dims_supplied} must be a permuted list of {dims_all}, unless `...` is included"
-            )
-        yield from existing_dims
-
-
 def get_temp_dimname(dims: Container[Hashable], new_dim: Hashable) -> Hashable:
     """Get an new dimension name based on new_dim, that is not used in dims.
     If the same name exists, we add an underscore(s) in the head.
@@ -884,72 +850,24 @@ def drop_dims_from_indexers(
         )
 
 
-def drop_missing_dims(
-    supplied_dims: Iterable[Hashable],
-    dims: Iterable[Hashable],
-    missing_dims: ErrorOptionsWithWarn,
-) -> Iterable[Hashable]:
-    """Depending on the setting of missing_dims, drop any dimensions from supplied_dims that
-    are not present in dims.
-
-    Parameters
-    ----------
-    supplied_dims : Iterable of Hashable
-    dims : Iterable of Hashable
-    missing_dims : {"raise", "warn", "ignore"}
-    """
-
-    if missing_dims == "raise":
-        supplied_dims_set = {val for val in supplied_dims if val is not ...}
-        invalid = supplied_dims_set - set(dims)
-        if invalid:
-            raise ValueError(
-                f"Dimensions {invalid} do not exist. Expected one or more of {dims}"
-            )
-
-        return supplied_dims
-
-    elif missing_dims == "warn":
-        invalid = set(supplied_dims) - set(dims)
-        if invalid:
-            warnings.warn(
-                f"Dimensions {invalid} do not exist. Expected one or more of {dims}"
-            )
-
-        return [val for val in supplied_dims if val in dims or val is ...]
-
-    elif missing_dims == "ignore":
-        return [val for val in supplied_dims if val in dims or val is ...]
-
-    else:
-        raise ValueError(
-            f"Unrecognised option {missing_dims} for missing_dims argument"
-        )
-
-
-T_None = TypeVar("T_None", None, "ellipsis")
-
-
 @overload
 def parse_dims(
-    dim: str | Iterable[Hashable] | T_None,
+    dim: Dims,
     all_dims: tuple[Hashable, ...],
     *,
     check_exists: bool = True,
     replace_none: Literal[True] = True,
-) -> tuple[Hashable, ...]:
-    ...
+) -> tuple[Hashable, ...]: ...
 
 
 @overload
 def parse_dims(
-    dim: str | Iterable[Hashable] | T_None,
+    dim: Dims,
     all_dims: tuple[Hashable, ...],
     *,
     check_exists: bool = True,
     replace_none: Literal[False],
-) -> tuple[Hashable, ...] | T_None:
-    ...
+) -> tuple[Hashable, ...] | None | ellipsis: ...
 
 
 def parse_dims(
@@ -995,28 +913,26 @@ def parse_dims(
 
 @overload
 def parse_ordered_dims(
-    dim: str | Sequence[Hashable | ellipsis] | T_None,
+    dim: Dims,
     all_dims: tuple[Hashable, ...],
     *,
     check_exists: bool = True,
     replace_none: Literal[True] = True,
-) -> tuple[Hashable, ...]:
-    ...
+) -> tuple[Hashable, ...]: ...
 
 
 @overload
 def parse_ordered_dims(
-    dim: str | Sequence[Hashable | ellipsis] | T_None,
+    dim: Dims,
     all_dims: tuple[Hashable, ...],
     *,
     check_exists: bool = True,
     replace_none: Literal[False],
-) -> tuple[Hashable, ...] | T_None:
-    ...
+) -> tuple[Hashable, ...] | None | ellipsis: ...
 
 
 def parse_ordered_dims(
-    dim: OrderedDims,
+    dim: Dims,
     all_dims: tuple[Hashable, ...],
     *,
     check_exists: bool = True,
@@ -1070,9 +986,9 @@ def parse_ordered_dims(
         )
 
 
-def _check_dims(dim: set[Hashable | ellipsis], all_dims: set[Hashable]) -> None:
-    wrong_dims = dim - all_dims
-    if wrong_dims and wrong_dims != {...}:
+def _check_dims(dim: set[Hashable], all_dims: set[Hashable]) -> None:
+    wrong_dims = (dim - all_dims) - {...}
+    if wrong_dims:
         wrong_dims_str = ", ".join(f"'{d!s}'" for d in wrong_dims)
         raise ValueError(
             f"Dimension(s) {wrong_dims_str} do not exist. Expected one or more of {all_dims}"
@@ -1094,12 +1010,10 @@ class UncachedAccessor(Generic[_Accessor]):
         self._accessor = accessor
 
     @overload
-    def __get__(self, obj: None, cls) -> type[_Accessor]:
-        ...
+    def __get__(self, obj: None, cls) -> type[_Accessor]: ...
 
     @overload
-    def __get__(self, obj: object, cls) -> _Accessor:
-        ...
+    def __get__(self, obj: object, cls) -> _Accessor: ...
 
     def __get__(self, obj: None | object, cls) -> type[_Accessor] | _Accessor:
         if obj is None:
@@ -1129,7 +1043,7 @@ def contains_only_chunked_or_numpy(obj) -> bool:
 
     Expects obj to be Dataset or DataArray"""
     from xarray.core.dataarray import DataArray
-    from xarray.core.pycompat import is_chunked_array
+    from xarray.namedarray.pycompat import is_chunked_array
 
     if isinstance(obj, DataArray):
         obj = obj._to_temp_dataset()
@@ -1142,31 +1056,13 @@ def contains_only_chunked_or_numpy(obj) -> bool:
     )
 
 
-def module_available(module: str) -> bool:
-    """Checks whether a module is installed without importing it.
-
-    Use this for a lightweight check and lazy imports.
-
-    Parameters
-    ----------
-    module : str
-        Name of the module.
-
-    Returns
-    -------
-    available : bool
-        Whether the module is installed.
-    """
-    return importlib.util.find_spec(module) is not None
-
-
 def find_stack_level(test_mode=False) -> int:
-    """Find the first place in the stack that is not inside xarray.
+    """Find the first place in the stack that is not inside xarray or the Python standard library.
 
     This is unless the code emanates from a test, in which case we would prefer
     to see the xarray source.
 
-    This function is taken from pandas.
+    This function is taken from pandas and modified to exclude standard library paths.
 
     Parameters
     ----------
@@ -1177,19 +1073,32 @@ def find_stack_level(test_mode=False) -> int:
     Returns
     -------
     stacklevel : int
-        First level in the stack that is not part of xarray.
+        First level in the stack that is not part of xarray or the Python standard library.
     """
     import xarray as xr
 
-    pkg_dir = os.path.dirname(xr.__file__)
-    test_dir = os.path.join(pkg_dir, "tests")
+    pkg_dir = Path(xr.__file__).parent
+    test_dir = pkg_dir / "tests"
+
+    std_lib_init = sys.modules["os"].__file__
+    # Mostly to appease mypy; I don't think this can happen...
+    if std_lib_init is None:
+        return 0
+
+    std_lib_dir = Path(std_lib_init).parent
 
-    # https://stackoverflow.com/questions/17407119/python-inspect-stack-is-slow
     frame = inspect.currentframe()
     n = 0
     while frame:
         fname = inspect.getfile(frame)
-        if fname.startswith(pkg_dir) and (not fname.startswith(test_dir) or test_mode):
+        if (
+            fname.startswith(str(pkg_dir))
+            and (not fname.startswith(str(test_dir)) or test_mode)
+        ) or (
+            fname.startswith(str(std_lib_dir))
+            and "site-packages" not in fname
+            and "dist-packages" not in fname
+        ):
             frame = frame.f_back
             n += 1
         else:
@@ -1204,11 +1113,11 @@ def emit_user_level_warning(message, category=None):
 
 
 def consolidate_dask_from_array_kwargs(
-    from_array_kwargs: dict,
+    from_array_kwargs: dict[Any, Any],
     name: str | None = None,
     lock: bool | None = None,
     inline_array: bool | None = None,
-) -> dict:
+) -> dict[Any, Any]:
     """
     Merge dask-specific kwargs with arbitrary from_array_kwargs dict.
 
@@ -1241,12 +1150,12 @@ def consolidate_dask_from_array_kwargs(
 
 
 def _resolve_doubly_passed_kwarg(
-    kwargs_dict: dict,
+    kwargs_dict: dict[Any, Any],
     kwarg_name: str,
     passed_kwarg_value: str | bool | None,
     default: bool | None,
     err_msg_dict_name: str,
-) -> dict:
+) -> dict[Any, Any]:
     # if in kwargs_dict but not passed explicitly then just pass kwargs_dict through unaltered
     if kwarg_name in kwargs_dict and passed_kwarg_value is None:
         pass
