@@ -9,8 +9,9 @@ from numpy.typing import ArrayLike
 from xarray.core import duck_array_ops, utils
 from xarray.core.alignment import align, broadcast
 from xarray.core.computation import apply_ufunc, dot
-from xarray.core.pycompat import is_duck_dask_array
-from xarray.core.types import Dims, T_Xarray
+from xarray.core.types import Dims, T_DataArray, T_Xarray
+from xarray.namedarray.utils import is_duck_dask_array
+from xarray.util.deprecation_helpers import _deprecate_positional_args
 
 # Weighted quantile methods are a subset of the numpy supported quantile methods.
 QUANTILE_METHODS = Literal[
@@ -144,7 +145,7 @@ class Weighted(Generic[T_Xarray]):
 
     __slots__ = ("obj", "weights")
 
-    def __init__(self, obj: T_Xarray, weights: DataArray) -> None:
+    def __init__(self, obj: T_Xarray, weights: T_DataArray) -> None:
         """
         Create a Weighted object
 
@@ -188,7 +189,7 @@ class Weighted(Generic[T_Xarray]):
             _weight_check(weights.data)
 
         self.obj: T_Xarray = obj
-        self.weights: DataArray = weights
+        self.weights: T_DataArray = weights
 
     def _check_dim(self, dim: Dims):
         """raise an error if any dimension is missing"""
@@ -198,19 +199,20 @@ class Weighted(Generic[T_Xarray]):
             dims = [dim] if dim else []
         else:
             dims = list(dim)
-        missing_dims = set(dims) - set(self.obj.dims) - set(self.weights.dims)
+        all_dims = set(self.obj.dims).union(set(self.weights.dims))
+        missing_dims = set(dims) - all_dims
         if missing_dims:
             raise ValueError(
-                f"{self.__class__.__name__} does not contain the dimensions: {missing_dims}"
+                f"Dimensions {tuple(missing_dims)} not found in {self.__class__.__name__} dimensions {tuple(all_dims)}"
             )
 
     @staticmethod
     def _reduce(
-        da: DataArray,
-        weights: DataArray,
+        da: T_DataArray,
+        weights: T_DataArray,
         dim: Dims = None,
         skipna: bool | None = None,
-    ) -> DataArray:
+    ) -> T_DataArray:
         """reduce using dot; equivalent to (da * weights).sum(dim, skipna)
 
         for internal use only
@@ -226,9 +228,9 @@ class Weighted(Generic[T_Xarray]):
 
         # `dot` does not broadcast arrays, so this avoids creating a large
         # DataArray (if `weights` has additional dimensions)
-        return dot(da, weights, dims=dim)
+        return dot(da, weights, dim=dim)
 
-    def _sum_of_weights(self, da: DataArray, dim: Dims = None) -> DataArray:
+    def _sum_of_weights(self, da: T_DataArray, dim: Dims = None) -> T_DataArray:
         """Calculate the sum of weights, accounting for missing values"""
 
         # we need to mask data values that are nan; else the weights are wrong
@@ -253,10 +255,10 @@ class Weighted(Generic[T_Xarray]):
 
     def _sum_of_squares(
         self,
-        da: DataArray,
+        da: T_DataArray,
         dim: Dims = None,
         skipna: bool | None = None,
-    ) -> DataArray:
+    ) -> T_DataArray:
         """Reduce a DataArray by a weighted ``sum_of_squares`` along some dimension(s)."""
 
         demeaned = da - da.weighted(self.weights).mean(dim=dim)
@@ -265,20 +267,20 @@ class Weighted(Generic[T_Xarray]):
 
     def _weighted_sum(
         self,
-        da: DataArray,
+        da: T_DataArray,
         dim: Dims = None,
         skipna: bool | None = None,
-    ) -> DataArray:
+    ) -> T_DataArray:
         """Reduce a DataArray by a weighted ``sum`` along some dimension(s)."""
 
         return self._reduce(da, self.weights, dim=dim, skipna=skipna)
 
     def _weighted_mean(
         self,
-        da: DataArray,
+        da: T_DataArray,
         dim: Dims = None,
         skipna: bool | None = None,
-    ) -> DataArray:
+    ) -> T_DataArray:
         """Reduce a DataArray by a weighted ``mean`` along some dimension(s)."""
 
         weighted_sum = self._weighted_sum(da, dim=dim, skipna=skipna)
@@ -289,10 +291,10 @@ class Weighted(Generic[T_Xarray]):
 
     def _weighted_var(
         self,
-        da: DataArray,
+        da: T_DataArray,
         dim: Dims = None,
         skipna: bool | None = None,
-    ) -> DataArray:
+    ) -> T_DataArray:
         """Reduce a DataArray by a weighted ``var`` along some dimension(s)."""
 
         sum_of_squares = self._sum_of_squares(da, dim=dim, skipna=skipna)
@@ -303,26 +305,27 @@ class Weighted(Generic[T_Xarray]):
 
     def _weighted_std(
         self,
-        da: DataArray,
+        da: T_DataArray,
         dim: Dims = None,
         skipna: bool | None = None,
-    ) -> DataArray:
+    ) -> T_DataArray:
         """Reduce a DataArray by a weighted ``std`` along some dimension(s)."""
 
-        return cast("DataArray", np.sqrt(self._weighted_var(da, dim, skipna)))
+        return cast("T_DataArray", np.sqrt(self._weighted_var(da, dim, skipna)))
 
     def _weighted_quantile(
         self,
-        da: DataArray,
+        da: T_DataArray,
         q: ArrayLike,
         dim: Dims = None,
         skipna: bool | None = None,
-    ) -> DataArray:
+    ) -> T_DataArray:
         """Apply a weighted ``quantile`` to a DataArray along some dimension(s)."""
 
         def _get_h(n: float, q: np.ndarray, method: QUANTILE_METHODS) -> np.ndarray:
             """Return the interpolation parameter."""
             # Note that options are not yet exposed in the public API.
+            h: np.ndarray
             if method == "linear":
                 h = (n - 1) * q + 1
             elif method == "interpolated_inverted_cdf":
@@ -448,18 +451,22 @@ class Weighted(Generic[T_Xarray]):
     def _implementation(self, func, dim, **kwargs):
         raise NotImplementedError("Use `Dataset.weighted` or `DataArray.weighted`")
 
+    @_deprecate_positional_args("v2023.10.0")
     def sum_of_weights(
         self,
         dim: Dims = None,
+        *,
         keep_attrs: bool | None = None,
     ) -> T_Xarray:
         return self._implementation(
             self._sum_of_weights, dim=dim, keep_attrs=keep_attrs
         )
 
+    @_deprecate_positional_args("v2023.10.0")
     def sum_of_squares(
         self,
         dim: Dims = None,
+        *,
         skipna: bool | None = None,
         keep_attrs: bool | None = None,
     ) -> T_Xarray:
@@ -467,9 +474,11 @@ class Weighted(Generic[T_Xarray]):
             self._sum_of_squares, dim=dim, skipna=skipna, keep_attrs=keep_attrs
         )
 
+    @_deprecate_positional_args("v2023.10.0")
     def sum(
         self,
         dim: Dims = None,
+        *,
         skipna: bool | None = None,
         keep_attrs: bool | None = None,
     ) -> T_Xarray:
@@ -477,9 +486,11 @@ class Weighted(Generic[T_Xarray]):
             self._weighted_sum, dim=dim, skipna=skipna, keep_attrs=keep_attrs
         )
 
+    @_deprecate_positional_args("v2023.10.0")
     def mean(
         self,
         dim: Dims = None,
+        *,
         skipna: bool | None = None,
         keep_attrs: bool | None = None,
     ) -> T_Xarray:
@@ -487,9 +498,11 @@ class Weighted(Generic[T_Xarray]):
             self._weighted_mean, dim=dim, skipna=skipna, keep_attrs=keep_attrs
         )
 
+    @_deprecate_positional_args("v2023.10.0")
     def var(
         self,
         dim: Dims = None,
+        *,
         skipna: bool | None = None,
         keep_attrs: bool | None = None,
     ) -> T_Xarray:
@@ -497,9 +510,11 @@ class Weighted(Generic[T_Xarray]):
             self._weighted_var, dim=dim, skipna=skipna, keep_attrs=keep_attrs
         )
 
+    @_deprecate_positional_args("v2023.10.0")
     def std(
         self,
         dim: Dims = None,
+        *,
         skipna: bool | None = None,
         keep_attrs: bool | None = None,
     ) -> T_Xarray:
