@@ -6,29 +6,31 @@ import pytest
 
 import xarray as xr
 from xarray.core import formatting_html as fh
+from xarray.core.coordinates import Coordinates
 
 
 @pytest.fixture
-def dataarray():
+def dataarray() -> xr.DataArray:
     return xr.DataArray(np.random.RandomState(0).randn(4, 6))
 
 
 @pytest.fixture
-def dask_dataarray(dataarray):
+def dask_dataarray(dataarray: xr.DataArray) -> xr.DataArray:
     pytest.importorskip("dask")
     return dataarray.chunk()
 
 
 @pytest.fixture
-def multiindex():
-    mindex = pd.MultiIndex.from_product(
+def multiindex() -> xr.Dataset:
+    midx = pd.MultiIndex.from_product(
         [["a", "b"], [1, 2]], names=("level_1", "level_2")
     )
-    return xr.Dataset({}, {"x": mindex})
+    midx_coords = Coordinates.from_pandas_multiindex(midx, "x")
+    return xr.Dataset({}, midx_coords)
 
 
 @pytest.fixture
-def dataset():
+def dataset() -> xr.Dataset:
     times = pd.date_range("2000-01-01", "2001-12-31", name="time")
     annual_cycle = np.sin(2 * np.pi * (times.dayofyear.values / 365.25 - 0.28))
 
@@ -46,17 +48,17 @@ def dataset():
     )
 
 
-def test_short_data_repr_html(dataarray) -> None:
+def test_short_data_repr_html(dataarray: xr.DataArray) -> None:
     data_repr = fh.short_data_repr_html(dataarray)
     assert data_repr.startswith("<pre>array")
 
 
-def test_short_data_repr_html_non_str_keys(dataset) -> None:
+def test_short_data_repr_html_non_str_keys(dataset: xr.Dataset) -> None:
     ds = dataset.assign({2: lambda x: x["tmin"]})
     fh.dataset_repr(ds)
 
 
-def test_short_data_repr_html_dask(dask_dataarray) -> None:
+def test_short_data_repr_html_dask(dask_dataarray: xr.DataArray) -> None:
     assert hasattr(dask_dataarray.data, "_repr_html_")
     data_repr = fh.short_data_repr_html(dask_dataarray)
     assert data_repr == dask_dataarray.data._repr_html_()
@@ -97,7 +99,7 @@ def test_summarize_attrs_with_unsafe_attr_name_and_value() -> None:
     assert "<dd>&lt;pd.DataFrame&gt;</dd>" in formatted
 
 
-def test_repr_of_dataarray(dataarray) -> None:
+def test_repr_of_dataarray(dataarray: xr.DataArray) -> None:
     formatted = fh.array_repr(dataarray)
     assert "dim_0" in formatted
     # has an expanded data section
@@ -119,12 +121,12 @@ def test_repr_of_dataarray(dataarray) -> None:
         )
 
 
-def test_repr_of_multiindex(multiindex) -> None:
+def test_repr_of_multiindex(multiindex: xr.Dataset) -> None:
     formatted = fh.dataset_repr(multiindex)
     assert "(x)" in formatted
 
 
-def test_repr_of_dataset(dataset) -> None:
+def test_repr_of_dataset(dataset: xr.Dataset) -> None:
     formatted = fh.dataset_repr(dataset)
     # coords, attrs, and data_vars are expanded
     assert (
@@ -151,7 +153,7 @@ def test_repr_of_dataset(dataset) -> None:
         assert "&lt;IA&gt;" in formatted
 
 
-def test_repr_text_fallback(dataset) -> None:
+def test_repr_text_fallback(dataset: xr.Dataset) -> None:
     formatted = fh.dataset_repr(dataset)
 
     # Just test that the "pre" block used for fallback to plain text is present.
@@ -170,7 +172,7 @@ def test_variable_repr_html() -> None:
     assert "xarray.Variable" in html
 
 
-def test_repr_of_nonstr_dataset(dataset) -> None:
+def test_repr_of_nonstr_dataset(dataset: xr.Dataset) -> None:
     ds = dataset.copy()
     ds.attrs[1] = "Test value"
     ds[2] = ds["tmin"]
@@ -179,7 +181,7 @@ def test_repr_of_nonstr_dataset(dataset) -> None:
     assert "<div class='xr-var-name'><span>2</span>" in formatted
 
 
-def test_repr_of_nonstr_dataarray(dataarray) -> None:
+def test_repr_of_nonstr_dataarray(dataarray: xr.DataArray) -> None:
     da = dataarray.rename(dim_0=15)
     da.attrs[1] = "value"
     formatted = fh.array_repr(da)
