@@ -739,7 +739,7 @@ def test_datetime_interp_noerror() -> None:
     xi = xr.DataArray(
         np.linspace(1, 3, 50),
         dims=["time"],
-        coords={"time": pd.date_range("01-01-2001", periods=50, freq="H")},
+        coords={"time": pd.date_range("01-01-2001", periods=50, freq="h")},
     )
     a.interp(x=xi, time=xi.time)  # should not raise an error
 
@@ -747,7 +747,7 @@ def test_datetime_interp_noerror() -> None:
 @requires_cftime
 @requires_scipy
 def test_3641() -> None:
-    times = xr.cftime_range("0001", periods=3, freq="500Y")
+    times = xr.cftime_range("0001", periods=3, freq="500YE")
     da = xr.DataArray(range(3), dims=["time"], coords=[times])
     da.interp(time=["0002-05-01"])
 
@@ -838,8 +838,8 @@ def test_interpolate_chunk_1d(
                         if chunked:
                             dest[dim] = xr.DataArray(data=dest[dim], dims=[dim])
                             dest[dim] = dest[dim].chunk(2)
-                actual = da.interp(method=method, **dest, kwargs=kwargs)  # type: ignore
-                expected = da.compute().interp(method=method, **dest, kwargs=kwargs)  # type: ignore
+                actual = da.interp(method=method, **dest, kwargs=kwargs)
+                expected = da.compute().interp(method=method, **dest, kwargs=kwargs)
 
                 assert_identical(actual, expected)
 
